Package: statcheck
Title: Extract Statistics from Articles and Recompute P-Values
Version: 1.5.0
Date: 2024-02-16
Authors@R: c(
    person("Michele B.", "Nuijten", , "m.b.nuijten@uvt.nl", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-1468-8585")),
    person("Sacha", "Epskamp", , "mail@sachaepskamp.com", role = "aut",
           comment = c(ORCID = "0000-0003-4884-8118")),
    person("Willem", "Sleegers", role = "ctb",
           comment = c(ORCID = "0000-0001-9058-3817")),
    person("Edoardo", "Costantini", role = "ctb"),
    person("Paul", "van der Laken", role = "ctb",
           comment = c(ORCID = "0000-0002-0404-9114")),
    person("Sean", "Rife", role = "ctb",
           comment = c(ORCID = "0000-0002-6748-0841")),
    person("John", "Sakaluk", role = "ctb",
           comment = c(ORCID = "0000-0002-2515-9822")),
    person("Chris", "Hartgerink", role = "ctb",
           comment = c(ORCID = "0000-0003-1050-6809")),
    person("Steve", "Haroz", role = "ctb",
           comment = c(ORCID = "0000-0002-2725-9173"))
  )
Description: A "spellchecker" for statistics. It checks whether your
    p-values match their accompanying test statistic and degrees of
    freedom. statcheck searches for null-hypothesis significance test
    (NHST) in APA style (e.g., t(28) = 2.2, p < .05). It recalculates the
    p-value using the reported test statistic and degrees of freedom. If
    the reported and computed p-values don't match, statcheck will flag
    the result as an error. If the reported p-value is statistically
    significant and the recomputed one is not, or vice versa, the result
    will be flagged as a decision error.  You can use statcheck directly
    on a string of text, but you can also scan a PDF or HTML file, or even
    a folder of PDF and/or HTML files.  Statcheck needs an external
    program to convert PDF to text: Xpdf. Instructions on where and how to
    download this program, how to install statcheck, and more details on
    what statcheck can and cannot do can be found in the online manual:
    <https://rpubs.com/michelenuijten/statcheckmanual>.  You can find a
    point-and-click web interface to scan PDF or HTML or DOCX articles on
    <http://statcheck.io>.
License: GPL-3
Depends: R (>= 2.14.2)
Imports: ggplot2, plyr, rlang, rmarkdown, stringi, tcltk
Suggests: testthat
ByteCompile: yes
Encoding: UTF-8
LazyLoad: yes
RoxygenNote: 7.3.1
URL: https://github.com/MicheleNuijten/statcheck
BugReports: https://github.com/MicheleNuijten/statcheck/issues
NeedsCompilation: no
Packaged: 2024-02-16 19:18:37 UTC; miche
Author: Michele B. Nuijten [aut, cre] (<https://orcid.org/0000-0002-1468-8585>),
  Sacha Epskamp [aut] (<https://orcid.org/0000-0003-4884-8118>),
  Willem Sleegers [ctb] (<https://orcid.org/0000-0001-9058-3817>),
  Edoardo Costantini [ctb],
  Paul van der Laken [ctb] (<https://orcid.org/0000-0002-0404-9114>),
  Sean Rife [ctb] (<https://orcid.org/0000-0002-6748-0841>),
  John Sakaluk [ctb] (<https://orcid.org/0000-0002-2515-9822>),
  Chris Hartgerink [ctb] (<https://orcid.org/0000-0003-1050-6809>),
  Steve Haroz [ctb] (<https://orcid.org/0000-0002-2725-9173>)
Maintainer: Michele B. Nuijten <m.b.nuijten@uvt.nl>
Repository: CRAN
Date/Publication: 2024-02-16 19:30:02 UTC
Built: R 4.6.0; ; 2026-04-26 15:21:05 UTC; unix

Package: statcheck
Title: Extract Statistics from Articles and Recompute P-Values
Version: 1.5.0
Date: 2024-02-16
Authors@R: c(
    person("Michele B.", "Nuijten", , "m.b.nuijten@uvt.nl", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-1468-8585")),
    person("Sacha", "Epskamp", , "mail@sachaepskamp.com", role = "aut",
           comment = c(ORCID = "0000-0003-4884-8118")),
    person("Willem", "Sleegers", role = "ctb",
           comment = c(ORCID = "0000-0001-9058-3817")),
    person("Edoardo", "Costantini", role = "ctb"),
    person("Paul", "van der Laken", role = "ctb",
           comment = c(ORCID = "0000-0002-0404-9114")),
    person("Sean", "Rife", role = "ctb",
           comment = c(ORCID = "0000-0002-6748-0841")),
    person("John", "Sakaluk", role = "ctb",
           comment = c(ORCID = "0000-0002-2515-9822")),
    person("Chris", "Hartgerink", role = "ctb",
           comment = c(ORCID = "0000-0003-1050-6809")),
    person("Steve", "Haroz", role = "ctb",
           comment = c(ORCID = "0000-0002-2725-9173"))
  )
Description: A "spellchecker" for statistics. It checks whether your
    p-values match their accompanying test statistic and degrees of
    freedom. statcheck searches for null-hypothesis significance test
    (NHST) in APA style (e.g., t(28) = 2.2, p < .05). It recalculates the
    p-value using the reported test statistic and degrees of freedom. If
    the reported and computed p-values don't match, statcheck will flag
    the result as an error. If the reported p-value is statistically
    significant and the recomputed one is not, or vice versa, the result
    will be flagged as a decision error.  You can use statcheck directly
    on a string of text, but you can also scan a PDF or HTML file, or even
    a folder of PDF and/or HTML files.  Statcheck needs an external
    program to convert PDF to text: Xpdf. Instructions on where and how to
    download this program, how to install statcheck, and more details on
    what statcheck can and cannot do can be found in the online manual:
    <https://rpubs.com/michelenuijten/statcheckmanual>.  You can find a
    point-and-click web interface to scan PDF or HTML or DOCX articles on
    <http://statcheck.io>.
License: GPL-3
Depends: R (>= 2.14.2)
Imports: ggplot2, plyr, rlang, rmarkdown, stringi, tcltk
Suggests: testthat
ByteCompile: yes
Encoding: UTF-8
LazyLoad: yes
RoxygenNote: 7.3.1
URL: https://github.com/MicheleNuijten/statcheck
BugReports: https://github.com/MicheleNuijten/statcheck/issues
NeedsCompilation: no
Packaged: 2024-02-16 19:18:37 UTC; miche
Author: Michele B. Nuijten [aut, cre] (<https://orcid.org/0000-0002-1468-8585>),
  Sacha Epskamp [aut] (<https://orcid.org/0000-0003-4884-8118>),
  Willem Sleegers [ctb] (<https://orcid.org/0000-0001-9058-3817>),
  Edoardo Costantini [ctb],
  Paul van der Laken [ctb] (<https://orcid.org/0000-0002-0404-9114>),
  Sean Rife [ctb] (<https://orcid.org/0000-0002-6748-0841>),
  John Sakaluk [ctb] (<https://orcid.org/0000-0002-2515-9822>),
  Chris Hartgerink [ctb] (<https://orcid.org/0000-0003-1050-6809>),
  Steve Haroz [ctb] (<https://orcid.org/0000-0002-2725-9173>)
Maintainer: Michele B. Nuijten <m.b.nuijten@uvt.nl>
Repository: CRAN
Date/Publication: 2024-02-16 19:30:02 UTC
Built: R 4.6.0; ; 2026-04-28 21:45:03 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(identify,statcheck)
S3method(plot,statcheck)
S3method(summary,statcheck)
export(checkHTML)
export(checkHTMLdir)
export(checkPDF)
export(checkPDFdir)
export(checkdir)
export(statcheck)
export(statcheckReport)
export(trim)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,plot.default)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(rlang,.data)

<!-- README.md is generated from README.Rmd. Please edit that file -->
<!-- after editing README.Rmd, run devtools::build_readme() -->

# statcheck <a href='http://statcheck.io'><img src='man/figures/logo.jpg' align="right" height="100" /></a>

<!-- badges: start -->

[![](https://www.r-pkg.org/badges/version/statcheck?color=green)](https://cran.r-project.org/package=statcheck)
[![](http://cranlogs.r-pkg.org/badges/grand-total/statcheck?color=green)](https://cran.r-project.org/package=statcheck)
[![](https://img.shields.io/badge/devel%20version-1.5.0-yellow.svg)](https://github.com/MicheleNuijten/statcheck)
<!-- badges: end -->

## What is statcheck?

`statcheck` is a “spellchecker” for statistics. It checks whether your
*p*-values match their accompanying test statistic and degrees of
freedom.

`statcheck` searches for null-hypothesis significance test (NHST) in APA
style (e.g., *t*(28) = 2.2, *p* \< .05). It recalculates the p-value
using the reported test statistic and degrees of freedom. If the
reported and computed p-values don’t match, `statcheck` will flag the
result as an error.

![](man/figures/infograph.png)

## What can I use statcheck for?

`statcheck` is mainly useful for:

1.  **Self-checks**: you can use `statcheck` to make sure your
    manuscript doesn’t contain copy-paste errors or other
    inconsistencies before you submit it to a journal.
2.  **Peer review**: editors and reviewers can use `statcheck` to check
    submitted manuscripts for statistical inconsistencies. They can ask
    authors for a correction or clarification before publishing a
    manuscript.
3.  **Research**: `statcheck` can be used to automatically extract
    statistical test results from articles that can then be analyzed.
    You can for instance investigate whether you can predict statistical
    inconsistencies (see e.g., Nuijten et al., 2017
    <doi:10.1525/collabra.102>), or use it to analyze p-value
    distributions (see e.g., Hartgerink et al., 2016
    <doi:10.7717/peerj.1935>).

## How does statcheck work?

The algorithm behind `statcheck` consists of four basic steps:

1.  **Convert** pdf and html articles to plain text files.
2.  **Search** the text for instances of NHST results. Specifically,
    `statcheck` can recognize *t*-tests, *F*-tests, correlations,
    *z*-tests, $\chi^2$ -tests, and Q-tests (from meta-analyses) if they
    are reported completely (test statistic, degrees of freedom, and
    *p*-value) and in APA style.
3.  **Recompute** the *p*-value using the reported test statistic and
    degrees of freedom.
4.  **Compare** the reported and recomputed *p*-value. If the reported
    *p*-value does not match the computed one, the result is marked as
    an *inconsistency* (`error` in the output). If the reported
    *p*-value is significant and the computed is not, or vice versa, the
    result is marked as a *gross inconsistency* (`decision_error` in the
    output).

`statcheck` takes into account correct rounding of the test statistic,
and has the option to take into account one-tailed testing. See the
[manual](http://rpubs.com/michelenuijten/statcheckmanual) for details.

## Installation and use

For detailed information about installing and using `statcheck`, see the
[manual on RPubs](http://rpubs.com/michelenuijten/statcheckmanual).

Also see [statcheck.io](http://statcheck.io/), a web-based interface for
statcheck.

# Inner function to read pdf:
getPDF <- function(x)
{
  txtfiles <- character(length(x))
  for (i in 1:length(x))
  {
    system(paste('pdftotext -q -enc "ASCII7" "', x[i], '"', sep = ""))
    if (file.exists(gsub("\\.pdf$", "\\.txt", x[i]))) {
      fileName <- gsub("\\.pdf$", "\\.txt", x[i])
      txtfiles[i] <- readChar(fileName, file.info(fileName)$size)
    } else{
      warning(paste("Failure in file", x[i]))
      txtfiles[i] <- ""
    }
  }
  return(txtfiles)
}

## Function to check directory of PDFs:
checkPDFdir <-
  function(dir,
           subdir = TRUE,
           ...) {
    if (missing(dir))
      dir <- tk_choose.dir()
    
    all.files <-
      list.files(dir,
                 pattern = "\\.pdf",
                 full.names = TRUE,
                 recursive = subdir)
    files <- all.files[grepl("\\.pdf$", all.files)]
    
    if (length(files) == 0)
      stop("No PDF found")
    
    txts <- character(length(files))
    message("Importing PDF files...")
    pb <- txtProgressBar(max = length(files), style = 3)
    for (i in 1:length(files))
    {
      txts[i] <-  getPDF(files[i])
      setTxtProgressBar(pb, i)
    }
    close(pb)
    names(txts) <- gsub("\\.pdf$", "", basename(files))
    return(statcheck(txts, ...))
  }

## Function to given PDFs:
checkPDF <-
  function(files, ...) {
    if (missing(files))
      files <- tk_choose.files()
    
    txts <-  sapply(files, getPDF)
    names(txts) <-
      gsub("\\.pdf$", "", basename(files), perl = TRUE)
    return(statcheck(txts, ...))
  }

context("Check if statistics from an article are correctly retrieved and parsed")

# tests concerning parsing stats from pdf files ------------------------------

# individual pdf files
test_that("statistics from a pdf are correctly retrieved and parsed", {
  
  pdf_file <- system.file("test_materials/nuijten.pdf",
                          package = "statcheck")
  
  # skip test if file is not available
  if(pdf_file == "") skip("Test file not available.")
  
  result <- checkPDF(pdf_file, messages = FALSE)
  result_1tailed <- checkPDF(pdf_file, messages = FALSE, OneTailedTxt = TRUE)
  
  # extract 4 tests from paper
  expect_equal(nrow(result), 8)
  expect_equal(as.character(result[[VAR_TYPE]]), c("t", "Chi2", "t", "F",
                                                   "F", "F", "Chi2", "t"))
  expect_equal(result[[VAR_TEST_VALUE]], c(-4.93, 6.9, 2, 1.203,
                                           12.03, 23.95, 6.53, 2))
  
  # check errors
  expect_equal(result[[VAR_ERROR]], c(FALSE, FALSE, FALSE, TRUE,
                                      FALSE, FALSE, FALSE, TRUE))
  expect_equal(result[[VAR_DEC_ERROR]], c(FALSE, FALSE, FALSE, TRUE,
                                          FALSE, FALSE, FALSE, TRUE))
  
  # check errors with one-tailed test detection
  expect_equal(result_1tailed[[VAR_ERROR]], c(FALSE, FALSE, FALSE, TRUE,
                                              FALSE, FALSE, FALSE, FALSE))
  expect_equal(result_1tailed[[VAR_DEC_ERROR]], c(FALSE, FALSE, FALSE, TRUE,
                                                  FALSE, FALSE, FALSE, FALSE))
})

# pdfs in folder
test_that("statistics from all pdfs in a folder are correctly retrieved and parsed", {
  
  pdf_folder <- system.file("test_materials", package = "statcheck")
  
  # skip test if files are not available
  if(!any(grepl(".pdf", list.files(pdf_folder)))) skip("Test files not available.")
  
  result <- checkPDFdir(pdf_folder, messages = FALSE, subdir = FALSE)
  
  # extract 53 tests from 4 papers
  expect_equal(nrow(result), 53)
  expect_equal(length(unique(result[[VAR_SOURCE]])), 4)
  
})

# tests concerning parsing stats from html files ------------------------------

# individual html files
test_that("statistics from a html are correctly retrieved and parsed", {
  
  html_file <- system.file("test_materials/nuijten.html",
                          package = "statcheck")
  
  # skip test if file is not available
  if(html_file == "") skip("Test file not available.")

  result <- checkHTML(html_file, messages = FALSE)
  result_1tailed <- checkHTML(html_file, messages = FALSE, OneTailedTxt = TRUE)
  
  # extract 6 tests from paper
  expect_equal(nrow(result), 6)
  expect_equal(as.character(result[[VAR_TYPE]]), c("t", "Chi2", "t", "F", "F", "t"))
  expect_equal(result[[VAR_TEST_VALUE]], c(-4.93, 6.9, 2, 1.203, 12.03, 2))
  
  # check errors
  expect_equal(result[[VAR_ERROR]], c(FALSE, FALSE, FALSE, TRUE, FALSE, TRUE))
  expect_equal(result[[VAR_DEC_ERROR]], c(FALSE, FALSE, FALSE, TRUE, FALSE, TRUE))
  
  # check errors with one-tailed test detection
  expect_equal(result_1tailed[[VAR_ERROR]], c(FALSE, FALSE, FALSE, TRUE, FALSE, FALSE))
  expect_equal(result_1tailed[[VAR_DEC_ERROR]], c(FALSE, FALSE, FALSE, TRUE, FALSE, FALSE))
  
})

# htmls in folder
test_that("statistics from all htmls in a folder are correctly retrieved and parsed", {
  
  html_dir <- system.file("test_materials", package = "statcheck")
  
  # skip test if files are not available
  if(!any(grepl(".htm*", list.files(html_dir)))) skip("Test files not available.")
  
  result <- checkHTMLdir(html_dir, messages = FALSE, subdir = FALSE)
  
  # extract 6+33 tests from papers 
  expect_equal(nrow(result), 39)
  
})

# tests concerning parsing stats from all files ------------------------------

# pdfs and htmls in folder
test_that("statistics from all pdfs and htmls in a folder are correctly retrieved and parsed", {
  
  dir <- system.file("test_materials", package = "statcheck")
  
  # skip test if files are not available
  if(!any(grepl(".htm*|.pdf", list.files(dir)))) skip("Test files not available.")
  
  result <- checkdir(dir, subdir = FALSE, messages = FALSE)
  
  # extract 92 tests (39 from html and 53 from pdf)
  expect_equal(nrow(result), 92)
})

# this script contains helper functions to extract and parse nhst results from 
# text

# function to extract snippets of text from a string ---------------------------

extract_pattern <- function(txt, pattern, ignore.case = TRUE) {
  
  # extract the locations of the matches in the text:
  # gregexpr returns the position of every match in a string
  # if there are multiple matches in the text, gregexpr will flag them all
  # the output is in list format, but the relevant information is all in [[1]]
  string_loc <- gregexpr(pattern = pattern, 
                         text = txt, 
                         ignore.case = ignore.case,
                         perl = TRUE)[[1]] # perl is necessary for lookbehinds
  
  # if no match is found, or result is a missing value, return NULL
  if(string_loc[1] == -1 | is.na(string_loc[1])){
    return(NULL)
  }
  
  
  # if a match is found:
  # extract the raw text of the regex match:
  # retrieve a 'substring' from the text that starts at string_loc: string_loc
  # is a vector of integers that indicate the location of the first characters of
  # the match. The end point of the substring is determined using the 
  # attribute 'match.length' of string_loc This is a vector of integers indicating
  # how long each matched string is. By adding the length of the string to the
  # position of the first character, and subtracting 1, you obtain the location
  # of the last character of the string.
  string <- substring(text = txt, 
                      first = string_loc, 
                      last = string_loc + attr(string_loc, "match.length") - 1)
  
  return(string)
}

# function to extract dfs from a raw nhst result -------------------------------

extract_df <- function(raw, test_type){
  
  # z tests do not have dfs, so return df1 = NA, and df2 = NA for z-tests
  if(test_type == "Z"){
    
    df1 <- NA
    df2 <- NA
    
  } else {
    # for all other test types, extract dfs from the raw nhst result
    df_raw <- extract_pattern(txt = raw,
                              pattern = RGX_DF)[[1]]
    
    # remove parentheses to only keep numbers
    df <- gsub("\\(|\\)", "", df_raw)
    
    # split string on comma to separate df1 and df2 / N
    # note: there can be no commas as thousand separators in the dfs; statcheck
    # would not have recognized these in the first place, so we don't have to take
    # this possibility into account here
    df <- strsplit(df, ",")[[1]]
    
    # remove leading/trailing whitespaces
    df <- trimws(df, which = "both")
    
    # there are three different types of degrees of freedom:
    # - t-tests, correlations, and Q-tests (a single number between brackets)
    # - F-tests (two degrees of freedom separated by a comma)
    # - chi2 (can also contain sample size)
    
    if(test_type %in% c("t", "r")){
      
      df1 <- NA
      df2 <- df
      
    } else if(test_type == "F"){
      
      # for some reason, typesetting in articles sometimes goes wrong with 
      # F-tests and when df1 == 1, it gets typeset as the letter l or I 
      # If this happens, replace the l or I with a 1
      
      if(grepl(RGX_DF1_I_L, df[1])){
        df[1] <- 1
      }
      
      df1 <- df[1]
      df2 <- df[2]
      
    } else if(test_type %in% c("Chi2", "Q", "Qw", "Qb")){
      
      df1 <- df[1]
      df2 <- NA
      
    } else {
      
      df1 <- NA
      df2 <- NA
      
    }
  }
  
  # return dfs as numeric values (to avoid R turning them into factors)
  return(data.frame(df1 = as.numeric(df1), 
                    df2 = as.numeric(df2)))
  
}

# function to remove commas that serve as thousands separators -----------------

remove_1000_sep <- function(raw){
  
  # replace all matches in the raw nhst results with nothing
  output <- gsub(pattern = RGX_1000_SEP,
                 replacement = "", 
                 x = raw,
                 perl = TRUE) # for the lookaheads & lookbehinds in the regex
  
  return(output)
  
}

# function to replace weird symbols with a minus sign --------------------------

# sometimes the mathematical symbol for a minus sign is wrongly converted into
# a strange symbol. Since it is very likely that any weird symbol in front of
# a test statistic is in fact a minus sign, replace all such weird codings with
# an actual minus sign

recover_minus_sign <- function(raw){
  
  # replace any weird string before the test value with a minus sign
  return(gsub(RGX_WEIRD_MINUS, " -", raw, perl = TRUE))
  
}



# function to extract test-values and test comparisons -------------------------

extract_test_stats <- function(raw){
  
  # remove N = ... from chi-square tests
  # otherwise, these sample sizes will wrongly be classified as test statistics
  raw_noN <- gsub(RGX_DF_CHI2, "", raw)
  
  # extract test comparison and test value
  test_raw <- extract_pattern(txt = raw_noN,
                              pattern = RGX_TEST_VALUE)
  
  # extract test comparison
  test_comp <- extract_pattern(txt = test_raw,
                               pattern = RGX_COMP)
  
  # remove test comparison to only keep numbers
  test_value <- gsub(RGX_COMP, "", test_raw)
  
  # remove thousand separators
  test_value <- remove_1000_sep(test_value)
  
  # replace weird coding before a test value with a minus sign
  test_value <- recover_minus_sign(test_value)
  
  # remove leading/trailing whitespaces 
  test_value <- trimws(test_value, which = "both")
  
  # remove comma at the end of the value
  test_value <- gsub(",$", "", test_value)
  
  # record the number of decimals of the test statistic
  test_dec <- attr(regexpr(RGX_DEC, test_value), "match.length") - 1
  test_dec[test_dec < 0] <- 0
  
  # make test_value numeric; suppress warnings (these could arive if the test
  # value is unusual, e.g., a weird minus followed by a space can't be made
  # numeric)
  # note: this needs to happen AFTER extracting the nr of decimals
  test_value <- suppressWarnings(as.numeric(test_value))
  
  return(data.frame(test_comp = test_comp,
                    test_value = test_value,
                    test_dec = test_dec,
                    stringsAsFactors = FALSE))
  
}

# function to extract and parse p-values --------------------------------------

extract_p_value <- function(raw){
  
  p_raw <- extract_pattern(txt = raw,
                           pattern = RGX_P_NS)
  
  p_comp <- character()
  p_value <- numeric()
  p_dec <- numeric()
  
  for(i in seq_along(p_raw)){
    
    if(grepl(RGX_NS, p_raw[i], ignore.case = TRUE)){
      
      p_comp[i] <- "ns"
      p_value[i] <- NA
      p_dec[i] <- NA
      
    } else {
      
      # extract p-comparison
      p_comp[i] <- extract_pattern(txt = p_raw[i],
                                pattern = RGX_COMP)
      
      # remove p comparison to only keep numbers
      # split the string on the comparison, that splits the string into a p and
      # the actual value. Only select the second element: the value
      p_value[i] <- strsplit(p_raw[i], RGX_COMP)[[1]][2]
      
      # remove leading/trailing whitespaces 
      p_value[i] <- trimws(p_value[i], which = "both")
      
      # record the number of decimals of the p-value
      dec <- attr(regexpr(RGX_DEC, p_value[i]), "match.length") - 1
      dec[dec < 0] <- 0
      
      p_dec[i] <- dec
      
    }
  }
  
  return(data.frame(p_comp = p_comp,
                    p_value = as.numeric(p_value),
                    p_dec = p_dec,
                    stringsAsFactors = FALSE))
  
}

#' @rdname checkdirs
#' @export

checkPDFdir <-
  function(dir,
           subdir = TRUE,
           ...) {
    if (missing(dir))
      dir <- tcltk::tk_choose.dir()
    
    all.files <-
      list.files(dir,
                 pattern = "\\.pdf",
                 full.names = TRUE,
                 recursive = subdir)
    files <- all.files[grepl("\\.pdf$", all.files)]
    
    if (length(files) == 0)
      stop("No PDF found")
    
    txts <- character(length(files))
    message("Importing PDF files...")
    pb <- utils::txtProgressBar(max = length(files), style = 3)
    
    for (i in 1:length(files)){
      txts[i] <-  getPDF(files[i])
      utils::setTxtProgressBar(pb, i)
    }
    
    close(pb)
    
    names(txts) <- basename(files)
    
    return(statcheck(txts, ...))
  }

#' Summary method for statcheck
#' 
#' Gives the summaries for a \code{statcheck} object.
#' 
#' @param object a \code{statcheck} object.
#' @param ... additional arguments affecting the summary produced.
#' 
#' @return A data frame containing for each source of statistics:
#' \describe{
#'     \item{source}{Name of the file/origin of which the statistics are 
#'     extracted}
#'     \item{nr_p_values}{The number of extracted reported p values per article}
#'     \item{nr_errors}{The number of errors per article}
#'     \item{nr_decision_errors}{The number of decision errors per article}
#' }
#' 
#' @examples 
#' txt <- "blablabla the effect was very significant (t(100)=1, p < 0.001)"
#' stat <- statcheck(txt)
#' summary(stat)
#' 
#' @export


summary.statcheck <-
  function(object, ...) {
    x <- object
    
    # Source
    Source <-
      c(as.vector(plyr::ddply(x, VAR_SOURCE, function(x)
        unique(x[[VAR_SOURCE]]))[, 1]), "Total")
    
    # Number of p values extracted per article and in total
    pValues <- c(plyr::ddply(x, VAR_SOURCE, function(x)
      nrow(x))[, 2], nrow(x))
    
    # Number of errors per article and in total
    Errors <-
      c(plyr::ddply(x, VAR_SOURCE, function(x)
        sum(x[[VAR_ERROR]], na.rm = TRUE))[, 2], 
        sum(x[[VAR_ERROR]], na.rm = TRUE))
    
    # Number of decision errors per article and in total
    DecisionErrors <-
      c(plyr::ddply(x, VAR_SOURCE, function(x)
        sum(x[[VAR_DEC_ERROR]], na.rm = TRUE))[, 2],
        sum(x[[VAR_DEC_ERROR]], na.rm = TRUE))
    
    # Results in dataframe
    res <- data.frame(
      Source = Source,
      pValues = pValues,
      Errors = Errors,
      DecisionErrors = DecisionErrors
    )
    
    # Rename columns based on constants in constants.R file
    colnames(res) <- c(VAR_SOURCE, VAR_NR_PVALUES, VAR_NR_ERRORS, 
                       VAR_NR_DEC_ERRORS)
    
    class(res) <- c("statcheck", "data.frame")
    
    return(res)
  }

context("Summarize statcheck output")

# Note: so far I haven't found a straightforward way to test plot.statcheck()
# and identify.statcheck()

# test summary.statcheck() -----------------------------------------------------
test_that("summary.statcheck() works for a single source", {
  
  txt1 <- "t(28) = 2.20, p = .06"
  txt2 <- "F(2, 28) = 2.20, p = .15"
  txt3 <- "r(28) = .22, p = .26"
  txt4 <- "chi2(28) = 22.20, p = .77"
  txt5 <- " z = 2.20, p = .04"
  txt6 <- "Q(28) = 22.20, p = .03"
  
  txt <- paste(txt1, txt2, txt3, txt4, txt5, txt6)
  result <- statcheck(txt, messages = FALSE)
  summary <- summary(result)
  
  expect_equal(nrow(summary), 2)
  expect_equal(as.vector(summary[[VAR_SOURCE]]), c("01", "Total"))
  expect_equal(summary[[VAR_NR_PVALUES]], c(6, 6))
  expect_equal(summary[[VAR_NR_ERRORS]], c(5, 5))
  expect_equal(summary[[VAR_NR_DEC_ERRORS]], c(2, 2))
  
})

test_that("summary.statcheck() works for multiple sources", {
  
  txt1 <- "t(28) = 2.20, p = .06"
  txt2 <- "F(2, 28) = 2.20, p = .15"
  txt3 <- "r(28) = .22, p = .26"
  txt4 <- "chi2(28) = 22.20, p = .77"
  txt5 <- " z = 2.20, p = .04"
  txt6 <- "Q(28) = 22.20, p = .03"
  
  result <- statcheck(c(txt1, txt2, txt3, txt4, txt5, txt6), messages = FALSE)
  summary <- summary(result)
  
  expect_equal(nrow(summary), 7)
  expect_equal(as.vector(summary[[VAR_SOURCE]]), c(as.character(1:6), "Total"))
  expect_equal(summary[[VAR_NR_PVALUES]], c(rep(1, 6), 6))
  expect_equal(summary[[VAR_NR_ERRORS]], c(1,1,1,0,1,1,5))
  expect_equal(summary[[VAR_NR_DEC_ERRORS]], c(1,0,0,0,0,1,2))
  
})

# technically, trim() is not an S3 method, but it serves the same function

test_that("trim() works on statcheck output", {
  
  txt <- "t(28) = 2.20, p = .06"
  result <- statcheck(txt, messages = FALSE)
  
  concise_output <- trim(result)
  
  colnames_concise <- c(VAR_SOURCE, VAR_RAW, VAR_COMPUTED_P,
                        VAR_ERROR, VAR_DEC_ERROR)
  
  expect_equal(colnames(concise_output), colnames_concise)
  expect_equal(nrow(result), nrow(concise_output))
  
})

extract_1tail <- function(txt){
  
    
    # search for "one-sided"/"one-tailed"/"directional" in full text to detect one-sided testing
    onesided <-
      gregexpr("one.?sided|one.?tailed|directional", txt, ignore.case = TRUE)[[1]]
    
    if (onesided[1] != -1) {
      onesided <- 1
    } else {
      onesided <- 0
    }
    
    OneTailedInTxt <- as.logical(onesided)
    
    return(OneTailedInTxt)
}

#' Generate HTML report for statcheck output
#' 
#' This function uses R Markdown to generate a nicely formatted HTML report of 
#' \code{\link{statcheck}} output.
#' 
#' This function temporarily saves the inserted \code{statcheck} output as an 
#' .RData file in the "output" folder in the statcheck package directory. This 
#' file is then called by the .Rmd template that is saved in the folder "rmd", 
#' also in the statcheck package directory. After the HTML report is generated, 
#' the .RData file is removed again.
#' 
#' @param statcheckOutput statcheck output of one of the following functions: 
#' \code{\link{statcheck}}, \code{\link{checkPDFdir}}, \code{\link{checkPDF}}, 
#' \code{\link{checkHTMLdir}}, \code{\link{checkHTML}}, or
#' \code{\link{checkdir}}.
#' @param outputFileName String specifying the file name under which you want to 
#' save the generated HTML report. The extension ".html" is automatically added, 
#' so doesn't need to be specified in this argument.
#' @param outputDir String specifying the directory in which you want to save 
#' the generated HTML report.
#' 
#' @return An HTML report, saved in the directory specified in the argument 
#' "outputDir".
#' 
#' @examples \dontrun{
#' 
#' # first generate statcheck output, for instance by using the statcheck() 
#' function
#' 
#' txt <- "blablabla the effect was very significant (t(100)=1, p < 0.001)"
#' stat <- statcheck(txt)
#' 
#' # next, use this output to generate a nice HTML report of the results
#' statcheckReport(stat, outputFileName="statcheckHTMLReport", 
#'                 outputDir="C:/mydocuments/results")
#' 
#' # you can now find your HTML report in the folder 
#' # "C:/mydocuments/results" under the name "statcheckHTMLReport.html".
#' 
#' }
#' 
#' @export

statcheckReport <-
  function(statcheckOutput,
           outputFileName,
           outputDir) {
    
    # set working directory to output file in statcheck package library
    currentWD <- getwd()
    setwd(system.file("inst/rmd", package = "statcheck"))
    
    # temporarily save statcheck output as RData in the selected working directory
    save(statcheckOutput, file = "statcheckOutput.RData")
    
    # run the markdown/knitr script
    statcheckReport_template <-
      system.file("rmd/statcheckReport_template.Rmd", package = "statcheck")
    rmarkdown::render(statcheckReport_template)
    
    # save/move the file in/to the specified output directory
    curDir <- system.file("rmd", package = "statcheck")
    file.rename(
      from = paste(curDir, "statcheckReport_template.html", sep = "/"),
      to = paste(outputDir, "/", outputFileName, ".html", sep = "")
    )
    
    # remove .RData file from package library folder
    file.remove(paste(curDir, "statcheckOutput.RData", sep = "/"))
    
    setwd(currentWD)
  }

decision_error_test <- function(reported_p, computed_p,
                              test_comparison, p_comparison,
                              alpha, pEqualAlphaSig){
  
  # replace 'ns' for > alpha -----------------------------------------------
  reported_p[p_comparison == "ns"] <- alpha
  p_comparison[p_comparison == "ns"] <- ">"
  
  # check errors for different combinations of <>= -------------------------
  
  # treat p = alpha as significant
  if(pEqualAlphaSig == TRUE){
    if(test_comparison == "="){
      
      if(p_comparison == "="){
        
        dec_error <- (reported_p <= alpha & computed_p > alpha) |
          (reported_p > alpha & computed_p <= alpha)
        
        return(dec_error)
        
      } else if(p_comparison == "<"){
        
        dec_error <- reported_p <= alpha & computed_p > alpha
        return(dec_error)
        
      } else if(p_comparison == ">"){
        
        dec_error <- reported_p >= alpha & computed_p <= alpha
        return(dec_error)
        
      }
      
    } else if(test_comparison == "<"){
      
      if(p_comparison == "="){
        dec_error <- reported_p <= alpha & computed_p >= alpha
        return(dec_error)
        
      } else if(p_comparison == "<"){
        
        dec_error <- reported_p <= alpha & computed_p >= alpha
        return(dec_error)
      } 
      else if(p_comparison == ">"){
        
        dec_error <- FALSE
        return(dec_error)
        
      }
      
    } else if(test_comparison == ">"){
      
      if(p_comparison == "="){
        
        dec_error <- reported_p > alpha & computed_p <= alpha
        return(dec_error)
        
      } else if(p_comparison == "<"){
        
        dec_error <- FALSE
        return(dec_error)
        
      } else if(p_comparison == ">"){
        
        dec_error <- reported_p >= alpha & computed_p <= alpha
        return(dec_error)
        
      }
      
    }
    
    return(NA)
    
    # treat p = alpha as significant
  } else if (pEqualAlphaSig == FALSE){
    
    if(test_comparison == "="){
      
      if(p_comparison == "="){
        
        dec_error <- (reported_p < alpha & computed_p >= alpha) |
          (reported_p >= alpha & computed_p < alpha)
        return(dec_error)
        
      } else if(p_comparison == "<"){
        
        dec_error <- reported_p <= alpha & computed_p >= alpha
        return(dec_error)
        
      } else if(p_comparison == ">"){
        
        dec_error <- reported_p >= alpha & computed_p < alpha
        return(dec_error)
        
      }
      
    } else if(test_comparison == "<"){
      
      if(p_comparison == "="){
        dec_error <- reported_p < alpha & computed_p >= alpha
        return(dec_error)
        
      } else if(p_comparison == "<"){
        
        dec_error <- reported_p <= alpha & computed_p >= alpha
        return(dec_error)
      } 
      else if(p_comparison == ">"){
        
        dec_error <- FALSE
        return(dec_error)
        
      }
      
    } else if(test_comparison == ">"){
      
      if(p_comparison == "="){
        
        dec_error <- reported_p >= alpha & computed_p <= alpha
        return(dec_error)
        
      } else if(p_comparison == "<"){
        
        dec_error <- FALSE
        return(dec_error)
        
      } else if(p_comparison == ">"){
        
        dec_error <- reported_p >= alpha & computed_p <= alpha
        return(dec_error)
        
      }
      
    }
    
    return(NA)
  }
}

#' Identify specific points in a statcheck plot.
#' 
#' With this function you can simply point and click on the datapoints in the 
#' plot to see the corresponding statcheck details, such as the paper from which 
#' the data came and the exact statistical results.
#' 
#' @inheritParams plot.statcheck
#'
#' @examples \dontrun{
#' 
#' # First we need a statcheck object
#' # Here, we create one by running statcheck on some raw text
#' 
#' txt <- "This test is consistent t(28) = 0.2, p = .84, but this one is 
#' inconsistent: F(2, 28) = 4.2, p = .01. This final test is even a
#' gross/decision inconsistency: z = 1.23, p = .03"
#' 
#' result <- statcheck(txt)
#' 
#' # Now, we can run identify.statcheck(), or shorter, simply identify():
#' identify(result)
#' 
#' # Further instructions:
#' # click on one or multiple points of interest
#' # press Esc
#' # a dataframe with information on the selected points will appear
#' 
#' }
#' 
#' @export

identify.statcheck <-
  function(x,
           alpha = .05,
           ...) {
    reported <- x[[VAR_REPORTED_P]]
    computed <- x[[VAR_COMPUTED_P]]
    
    # replace 'ns' for > alpha
    reported[x[[VAR_P_COMPARISON]] == "ns"] <- alpha
    
    plot.statcheck(x, APAstyle = FALSE, ...) 
    ID <- graphics::identify(reported, computed)
    
    res <- x[ID,]
    class(res) <- c("statcheck", "data.frame")
    
    return(res)
  }

getHTML <- function(x){
  
  strings <- lapply(x, function(fileName)readChar(file(fileName), file.info(fileName)$size, useBytes = TRUE))
  
  # Remove subscripts (except for p_rep)
  strings <- lapply(strings, gsub, pattern = "<sub>(?!rep).*?</sub>", replacement = "", perl = TRUE)
  
  # Remove HTML tags:
  strings <- lapply(strings, gsub, pattern = "<(.|\n)*?>", replacement = "")
  
  # Replace html codes:
  strings <- lapply(strings, gsub, pattern = "&#60;", replacement = "<", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&lt;", replacement = "<", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#61;", replacement = "=", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#62;", replacement = ">", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&gt;", replacement = ">", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#40;", replacement = "(", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#41;", replacement = ")", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&thinsp;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&nbsp;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "\n", replacement = "")
  strings <- lapply(strings, gsub, pattern = "\r", replacement = "")
  strings <- lapply(strings, gsub, pattern = "\\s+", replacement = " ")
  strings <- lapply(strings, gsub, pattern = "&minus;", replacement = "-", fixed = TRUE)
  
  return(strings)
}




checkHTMLdir <- function(dir,
                         subdir = TRUE,
                         extension = TRUE,
                         ...) {
  if (missing(dir)) {
    dir <- tk_choose.dir()
  }
  
  if (extension == TRUE) {
    pat = ".html|.htm"
  }
  
  if (extension == FALSE) {
    pat = ""
  }
  
  files <-
    list.files(dir,
               pattern = pat,
               full.names = TRUE,
               recursive = subdir)
  
  if (length(files) == 0) {
    stop("No HTML found")
  }
  
  txts <- character(length(files))
  message("Importing HTML files...")
  pb <- txtProgressBar(max = length(files), style = 3)
  
  for (i in 1:length(files)) {
    txts[i] <-  getHTML(files[i])
    setTxtProgressBar(pb, i)
  }
  
  close(pb)
  
  names(txts) <- gsub(".html", "", basename(files))
  names(txts) <- gsub(".htm", "", names(txts))
  return(statcheck(txts, ...))
}

checkHTML <- function(files,
                      ...)
{
  if (missing(files))
    files <- tk_choose.files()
  
  txts <-  sapply(files, getHTML)
  names(txts) <- gsub(".html", "", basename(files))
  names(txts) <- gsub(".htm", "", names(txts))
  return(statcheck(txts, ...))
  
}

#' @rdname checkfiles
#' @export

checkHTML <- function(files,
                      ...)
{
  if (missing(files))
    files <- tcltk::tk_choose.files()
  
  txts <-  sapply(files, getHTML)
  names(txts) <- basename(files)
  return(statcheck(txts, ...))
  
}

compute_p <- function(test_type, test_stat, df1, df2, two_tailed){
  
  stopifnot(test_type %in% c("t", "F", "Z", "r", "Chi2", "Q", "Qb", "Qw"))
  
  # compute p-values ---------------------------------------------------------
  
  if(test_type == "t"){
    
    computed <- stats::pt(-1 * abs(test_stat), df2)
    
  } else if(test_type == "F"){
    
    computed <- stats::pf(test_stat, df1, df2, lower.tail = FALSE)
    
  } else if(test_type == "Z"){
    
    computed <- stats::pnorm(abs(test_stat), lower.tail = FALSE)
    
  } else if(test_type == "r"){
    
    t <- r2t(test_stat, df2)
    computed <- stats::pt(-1 * abs(t), df2)
    
  } else if(test_type == "Chi2" | 
            test_type == "Q" | test_type == "Qb" | test_type == "Qw"){
    
    computed <- stats::pchisq(test_stat, df1, lower.tail = FALSE)
    
  }
  
  # compute two-tailed ------------------------------------------------------
  
  if (!is.na(computed) &
    (test_type == "t" | test_type == "Z" | test_type == "r") & 
    two_tailed) {
    computed <- computed * 2
  }
  
  # return ------------------------------------------------------------------
  
  return(computed)
}


# Function to transform correlations into t-values by use of raw r and degrees of freedom.
r2t <- function(r, df){
  t <- r / (sqrt((1 - r^2) / df))
  return(t)
}

context("Correctly recalculate p-values")

test_that("p-values for t-tests are correctly calculated", {
  txt <- "t(28) = 2.20, p = .03"
  computed <- pt(-1 * abs(2.20), 28) * 2
  
  result <- statcheck(txt, messages = FALSE)
  
  expect_equal(result[[VAR_COMPUTED_P]], computed)
})


test_that("p-values for F-tests are correctly calculated", {
  txt <- "F(2, 28) = 2.20, p = .15"  
  computed <- pf(2.20, 2, 28, lower.tail = FALSE)
  
  result <- statcheck(txt, messages = FALSE)
  
  expect_equal(result[[VAR_COMPUTED_P]], computed)
})

test_that("p-values for correlations are correctly calculated", {
  txt <- "r(28) = .22, p = .26"
  
  pComputed <-
    pmin(pt(-1 * abs(r2t(.22, 28)), 28) * 2, 1)
  pComputed[is.nan(pComputed)] <- NA
  computed <-  pComputed
  
  result <- statcheck(txt, messages = FALSE)
  
  expect_equal(result[[VAR_COMPUTED_P]], computed)
})


test_that("p-values for z-tests are correctly calculated", {
  txt <- " z = 2.20, p = .04"
  computed <- pnorm(abs(2.20), lower.tail = FALSE) * 2
  
  result <- statcheck(txt, messages = FALSE)
  
  expect_equal(result[[VAR_COMPUTED_P]], computed)
})


test_that("p-values for chi2-tests are correctly calculated", {
  txt <- "chi2(28) = 22.20, p = .79"
  computed <- pchisq(22.20, 28, lower.tail = FALSE)
  
  result <- statcheck(txt, messages = FALSE)
  
  expect_equal(result[[VAR_COMPUTED_P]], computed)
})


test_that("p-values for Q-tests are correctly calculated", {
  txt <- "Q(28) = 22.20, p = .79"
  computed <- pchisq(22.20, 28, lower.tail = FALSE)
  
  result <- statcheck(txt, messages = FALSE)
  
  expect_equal(result[[VAR_COMPUTED_P]], computed)
})

#' statcheck: Extract statistics from articles and recompute p-values
#' 
#' The package \code{statcheck} can extract Null Hypothesis Significance Test
#' (NHST) results from articles (or plain text) and recomputes p-values to check 
#' whether a reported NHST result is internally consistent or not. 
#' 
#' \code{statcheck} can be used for multiple purposes, including:
#' 
#' \itemize{
#'     \item \strong{Self-checks}: you can use statcheck to make sure your 
#'     manuscript doesn't contain copy-paste errors or other inconsistencies 
#'     before you submit it to a journal.
#'     \item \strong{Peer review}: editors and reviewers can use statcheck to 
#'     check submitted manuscripts for statistical inconsistencies. They can ask 
#'     authors for a correction or clarification before publishing a manuscript.
#'     \item \strong{Research}: statcheck can be used to automatically extract 
#'     statistical test results from articles that can then be analyzed. You can 
#'     for instance investigate whether you can predict statistical 
#'     inconsistencies (see e.g., Nuijten et al., 2017 <doi:10.1525/collabra.102>), 
#'     or use it to analyze p-value distributions (see e.g., 
#'     Hartgerink et al., 2016 <doi:10.7717/peerj.1935>).
#' }
#' 
#' @section Using statcheck on a string of text:
#' The most basic usage of \code{statcheck} is to directly extract NHST results 
#' and check for inconsistencies in a string of text. See 
#' \code{\link{statcheck}} for details and an example of how to do this.
#' 
#' @section Using statcheck on an article:
#' Another option is to run \code{statcheck} on an article (PDF or HTML). This 
#' is a useful option if you want to check for inconsistencies in a single 
#' article (e.g., as a final check before you submit it). Depending on whether
#' you want to check an article in HTML or PDF, you can use 
#' \code{\link{checkHTML}} or \code{\link{checkPDF}}, respectively. Note: it is
#' recommended to check articles in HTML, as converting PDF files to plain text
#' sometimes results in some conversion errors.
#' 
#' @section Using statcheck on a folder of articles:
#' Finally, it is possible to run \code{statcheck} on an entire folder of 
#' articles. This is often useful for meta-research. To do so, you can use
#' \code{\link{checkPDFdir}} to check all PDF articles in a folder, 
#' \code{\link{checkHTMLdir}} to check all PDF articles in a folder, and 
#' \code{\link{checkdir}} to check both PDF and HTML articles in a folder.
#' 
#' @section Accuracy of the algorithm in detecting inconsistencies:
#' It is important to note that \code{statcheck} is not perfect. Its performance
#' in detecting NHST results depends on the type-setting and reporting style of 
#' an article and can vary widely. However, \code{statcheck} performs well in 
#' classifying the retrieved statistics in different consistency categories. We 
#' found that statcheck’s sensitivity (true positive rate) and specificity (true 
#' negative rate) were high: between 85.3% and 100%, and between 96.0% and 100%, 
#' respectively, depending on the assumptions and settings. The overall accuracy 
#' of statcheck ranged from 96.2% to 99.9%. More details on the validity study
#' can be found in \href{https://psyarxiv.com/tcxaj/}{Nuijten et al., 2017}.
#' 
#' @section Manual: 
#' Details on what statcheck can and cannot do, and how to install the package
#' and the necessary program Xpdf can be found in the 
#' \href{https://rpubs.com/michelenuijten/statcheckmanual}{online manual}.
#' 
#' @section Web app:
#' \code{statcheck} is also available as a free, online web app at 
#' \url{http://statcheck.io}. 
#' 
#' @references 
#' Hartgerink, C. H. J., Van Aert, R. C. M., Nuijten, M. B., Wicherts, J. M., 
#' Van Assen, M. A. L. M. (2016). Distributions of p-values smaller than .05 in 
#' psychology: What is going on? \emph{PeerJ}, \emph{4}, e1935. 
#' doi: 10.7717/peerj.1935
#' 
#' Nuijten, M. B., Borghuis, J., Veldkamp, C. L. S., Dominguez-Alvarez, L., Van 
#' Assen, M. A. L. M., & Wicherts, J. M. (2017).  Journal data sharing policies 
#' and statistical reporting inconsistencies in psychology. 
#' \emph{Collabra: Psychology}, \emph{3}(1), 1-22. doi: 10.1525/collabra.102.
#' 
#' Nuijten, M. B., Van Assen, M. A. L. M., Hartgerink, C. H. J., Epskamp, S., & 
#' Wicherts, J. M. (2017). The validity of the tool "statcheck" in discovering 
#' statistical reporting inconsistencies. \emph{Preprint retrieved from 
#' https://psyarxiv.com/tcxaj/.}

#' @keywords internal
#' @aliases statcheck-package
"_PACKAGE"

#' @name statcheck


# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
## usethis namespace: end
NULL

#' Trimming method for statcheck output
#' 
#' Returns a subset of columns of a \code{statcheck} object.
#' 
#' @param object a \code{statcheck} object.
#' @param ... additional arguments affecting the trimmed output.
#' 
#' @return A data frame containing for each source of statistics:
#' \describe{
#'     \item{source}{Name of the file/origin of which the statistics are 
#'     extracted}
#'     \item{raw}{Raw string of the statistical reference that is extracted}
#'     \item{computed_p}{The recomputed p-value}
#'     \item{error}{The computed p value is not congruent with the reported 
#'     p-value}
#'     \item{decision_error}{The reported result is significant whereas the 
#'     recomputed result is not, or vice versa.}
#' }
#' 
#' @examples 
#' txt <- "blablabla the effect was very significant (t(100)=1, p < 0.001)"
#' stat <- statcheck(txt)
#' trim(stat)
#' 
#' @export

trim <- 
  function(object, ...){
   
    x <- object
    
    colnames_concise <- c(VAR_SOURCE, VAR_RAW, VAR_COMPUTED_P,
                          VAR_ERROR, VAR_DEC_ERROR)
    
    concise_output <- x[ , colnames_concise]
        
    class(concise_output) <- c("statcheck", "data.frame")
    
    return(concise_output)
  }

# this script contains the regular expressions that statcheck uses to extract
# NHST results from text

# test types
RGX_T <- "t"
RGX_R <- "r"
RGX_Q <- "Q\\s?-?\\s?(w|W|(w|W)ithin|b|B|(b|B)etween)?"
RGX_F <- "F"
RGX_Z <- "([^a-z](z|Z))"
# for chi2: effectively extract everything that is NOT a t, r, F, z, Q, W, n, or 
# D, followed by *maybe* a 2 (and later followed by a result in a chi2 layout)
RGX_CHI2 <- "((\\s[^trFzZQWnD ]\\s?)|([^trFzZQWnD ]2\\s?))2?"

# degrees of freedom
# the way dfs are reported differs per test type, except for t, r, and Q, where
# they are always in the format "(28)". The regex for these tests can therefore
# be combined
# z-tests do not have dfs
RGX_DF_T_R_Q <- "\\(\\s?\\d*\\.?\\d+\\s?\\)"
RGX_DF_F <- "\\(\\s?\\d*\\.?(I|l|\\d+)\\s?,\\s?\\d*\\.?\\d+\\s?\\)"
RGX_DF_CHI2 <- "\\(\\s?\\d*\\.?\\d+\\s?(,\\s?(N|n)\\s?\\=\\s?\\d*\\,?\\d*\\,?\\d+\\s?)?\\)"

# combine test types with the correct type of df
# put regex between () to create regex groups
RGX_T_DF <- paste0("(", RGX_T, "\\s?", RGX_DF_T_R_Q, ")")
RGX_R_DF <- paste0("(", RGX_R, "\\s?", RGX_DF_T_R_Q, ")")
RGX_Q_DF <- paste0("(", RGX_Q, "\\s?", RGX_DF_T_R_Q, ")")
RGX_F_DF <- paste0("(", RGX_F, "\\s?", RGX_DF_F, ")")
RGX_CHI2_DF <- paste0("(", RGX_CHI2, "\\s?", RGX_DF_CHI2, ")")

RGX_TEST_DF <- paste0("(", RGX_T_DF, "|", RGX_R_DF, "|", RGX_Q_DF, "|", RGX_F_DF, 
                      "|", RGX_CHI2_DF, "|", RGX_Z, ")")

# test value
# this is the same for every type of test
# the part "[^a-zA-Z\\d\\.]{0,3}" is to extract punctuation marks that could 
# signal a weirdly encoded minus sign
# note that impossible values such as r > 1 are excluded at a later stage
RGX_TEST_VALUE <- "[<>=]\\s?[^a-zA-Z\\d\\.]{0,3}\\s?\\d*,?\\d*\\.?\\d+\\s?,"

# p-values
# this is the same for every type of test
RGX_NS <- "([^a-z]n\\.?s\\.?)"
RGX_P <- "(p\\s?[<>=]\\s?\\d?\\.\\d+e?-?\\d*)"
RGX_P_NS <- paste0("(", RGX_NS, "|", RGX_P, ")")

# full result
RGX_NHST <- paste(RGX_TEST_DF, RGX_TEST_VALUE, RGX_P_NS, sep = "\\s?")

################################################################################

# regex to recognize test type

# match everything up until the first occurence of a "(" with a positive look
# ahead. A "(" signals the start of the degrees of freedom, so everything before
# that should be the test statistic. Also match the regex for a z-test 
# (because a z-test has no df)

RGX_OPEN_BRACKET <- "(.+?(?=\\())"
RGX_TEST_TYPE <- paste(RGX_Z, RGX_OPEN_BRACKET, sep = "|")

# regex for Q-test

# for the Q-test, we also need to distinguish between Q, Qw, and Qb
# select all raw_nhst results that seem to have a Q-test in them
# it suffices to simply search for the letters "w" and "b"
RGX_QW <- "w"
RGX_QB <- "b"

# regex for degrees of freedom

# combine the separate regexes for the different types of dfs
# in one all-encompassing regex. Group the df-types with parentheses and 
# separate with an OR sign
RGX_DF <- paste0("(", RGX_DF_T_R_Q, ")|(", RGX_DF_F, ")|(", RGX_DF_CHI2, ")")

# regex for comparison symbols
RGX_COMP <- "[<>=]"

# regex for thousands separator
# this regex matches commas flanked by digits on both sides
RGX_1000_SEP <- "(?<=\\d),(?=\\d+)"

# regex for numbers after a point
# used to determine number of decimals
RGX_DEC <- "\\.\\d+"

# regex for weird symbols that should be a minus sign
# match potentially a space, followed by one or more characters that are not a 
# digit, period, or space, followed by a digit or period (using a positive 
# lookahead)
RGX_WEIRD_MINUS <- "\\s?[^\\d\\.\\s]+(?=\\d|\\.)"

# regex for weird df1 in F-tests
# for some reason, typesetting in articles sometimes goes wrong with 
# F-tests and when df1 == 1, it gets typeset as the letter l or I 
RGX_DF1_I_L <- "I|l"

#' Extract statistics from PDF/HTML articles and recalculate p-values
#' 
#' These functions search for NHST results in PDF and/or HTML articles and send 
#' the extracted statistics to \code{statcheck}.
#' 
#' @param files Vector of strings containing file paths to HTML files to check.
#' @param ... Arguments sent to \code{statcheck}.
#' 
#' @name checkfiles
#' 
#' @return A statcheck data frame with the extracted statistics. See 
#' \code{\link{statcheck}} for details.

NULL

extract_stats <- function(txt, stat){
  
  # step 1: extract all NHST results from text -----------------------------------
  
  # if there are x NHST results in a text, this returns a character vector of 
  # length x
  nhst_raw <- extract_pattern(txt = txt,
                              # nhst is the regex for nhst results
                              # it came from the regex.R script within the package
                              pattern = RGX_NHST,
                              # don't ignore case here: we only want to extract
                              # stats reported in a specific way (e.g., t, not T)
                              ignore.case = FALSE) 
  
  # if there are no nhst results in the text, return an empty data frame
  if(is.null(nhst_raw)){
    return(data.frame(NULL))
  }
  
  # step 2: parse the extracted results ------------------------------------------
  
  # empty vectors and data frames to store results in
  # where relevant, force the output to be of type "character"
  test_type <- character()
  df_result <- data.frame(NULL)  # will contain both df1 and df2
  test_stats <- data.frame(NULL) # will contain both test comparison and value
  pvals <- data.frame(NULL)      # will contain both p-comparison and p-value
  
  # loop over all extracted, raw nhst results
  # each nhst result is parsed individually, in the order that they were extracted
  # from the text
  for(i in seq_along(nhst_raw)){
    
    # extract the test types from the nhst results 
    test_raw <- extract_pattern(txt = nhst_raw[i],
                                pattern = RGX_TEST_TYPE)
    
    # classify the test types in standard classifications
    
    # for each test type, check where the vector with extracted, raw test types
    # matches the regex for each test type, and assign the appropriate category
    # the order of the classification matters: Q has to be tested first, 
    # otherwise the regex for t will also match Qwithin and Qbetween, because
    # both have a t in them. Similarly: first check for Qb, because Qbetween
    # also has a w in it
    if (grepl(pattern = RGX_Q, x = test_raw)){
      
      # distinguish between Q, Qw, and Qb
      if(grepl(pattern = RGX_QB, x = test_raw)){
        test_type[i] <- "Qb"
      } else if (grepl(pattern = RGX_QW, x = test_raw)){
        test_type[i] <- "Qw"
      } else {
        test_type[i] <- "Q"
      }
    } else if(grepl(pattern = RGX_T, x = test_raw)){
      test_type[i] <- "t"
    } else if (grepl(pattern = RGX_F, x = test_raw)){
      test_type[i] <- "F"
    } else if (grepl(pattern = RGX_R, x = test_raw)){
      test_type[i] <- "r"
    } else if (grepl(pattern = RGX_Z, x = test_raw)){
      test_type[i] <- "Z"
    } else if (grepl(pattern = RGX_CHI2, x = test_raw)){
      test_type[i] <- "Chi2"
    }
    
    # extract degrees of freedom
    
      dfs <- extract_df(raw = nhst_raw[i],
                        test_type = test_type[i])
      
      df_result <- rbind(df_result, dfs)
    
    # extract test comparison and test value 
    
    test <- extract_test_stats(raw = nhst_raw[i])
    
    # ignore any cases with more than 1 test comparison (e.g., z >= 2)
    if(nrow(test) > 1){
      test <- data.frame(test_comp = NA,
                         test_value = NA,
                         test_dec = NA)
    } 
    
    test_stats <- rbind(test_stats, test)  
    
    # extract p-comparison and p-value
    
    p <- extract_p_value(raw = nhst_raw[i])
    
    # ignore any cases with more than 1 test comparison (e.g., p >= .01)
    if(nrow(p) > 1){
      p <- data.frame(p_comp = NA,
                      p_value = NA,
                      p_dec = NA)
    }
    
    pvals <- rbind(pvals, p)
    
    
  }
  
  # create final data frame ------------------------------------------------------
  
  nhst_parsed <- data.frame(
    # return raw result without leading/trailing whitespaces
    Raw = trimws(nhst_raw, which = "both"), 
    Statistic = test_type,
    df1 = df_result$df1,
    df2 = df_result$df2,
    Test.Comparison = test_stats$test_comp,
    Value = test_stats$test_value,
    testdec = test_stats$test_dec,
    Reported.Comparison = pvals$p_comp,
    Reported.P.Value = pvals$p_value,
    dec = pvals$p_dec,
    stringsAsFactors = FALSE)
  
  if (nrow(nhst_parsed) > 0) {
  
    # remove p values greater than one
    nhst_parsed <- nhst_parsed[nhst_parsed$Reported.P.Value <= 1 |
                                 is.na(nhst_parsed$Reported.P.Value), ]
    
    # remove correlations greater than one or smaller than -1
    # reason: there is a risk that statcheck simply misread this stat
    # this means that statcheck will not flag such incorrect correlations,
    # which you could also consider a disadvantage
    nhst_parsed <- nhst_parsed[!(nhst_parsed$Statistic == "r" & 
                                 (nhst_parsed$Value > 1|
                                   nhst_parsed$Value < -1)), ]
    
    # remove rows with missing test values
    # reason: this can happen when a test statistic has a weird minus and a 
    # space in front of it. statcheck can't convert the weird minus in that case
    # and would otherwise break down
    nhst_parsed <- nhst_parsed[!is.na(nhst_parsed$Value), ]
    
    # remove missing test comparisons or p comparisons 
    # reason: these are marked as NA in cases where multiple comparisons were
    # reported. E.g., t(23) >= ..., p >= ...
    nhst_parsed <- nhst_parsed[!is.na(nhst_parsed$Test.Comparison) &
                                 !is.na(nhst_parsed$Reported.Comparison), ]
    
    # only return selected stats
    # to that end, rename test-types to match argument stat
    types <- as.vector(nhst_parsed$Statistic)
    
    types[types == "r"] <- "cor"
    types[types == "Chi2"] <- "chisq"
    types[types == "Z"] <- "Z"
    types[types == "Qw"| types == "Qb"] <- "Q"
    
    # only return rows where the test_type matches the selected stats
    nhst_parsed <- nhst_parsed[types %in% stat, ]
    
  }
  
  class(nhst_parsed) <- c("statcheck", "data.frame")
  
  return(nhst_parsed)
}

# This file contains constants. These are mainly fixed variable names used
# throughout the package
# The logic of having a "constants" file, is that it now becomes straightforward
# to change a variable name, without having to go through all R scripts

# variables in final statcheck data frame
VAR_SOURCE <- "source"
VAR_TYPE <- "test_type"
VAR_DF1 <- "df1"
VAR_DF2 <- "df2"
VAR_TEST_COMPARISON <- "test_comp"
VAR_TEST_VALUE <- "test_value"
VAR_P_COMPARISON <- "p_comp"
VAR_REPORTED_P <- "reported_p"
VAR_COMPUTED_P <- "computed_p"
VAR_RAW <- "raw"
VAR_ERROR <- "error"
VAR_DEC_ERROR <- "decision_error"
VAR_1TAILTXT <- "one_tailed_in_txt"
VAR_APAFACTOR <- "apa_factor"

# additional variables to return
VAR_P_DEC <- "p_decimals"

# additional variables in summary function
VAR_NR_PVALUES <- "nr_p_values"
VAR_NR_ERRORS <- "nr_errors"
VAR_NR_DEC_ERRORS <- "nr_decision_errors"

#' @rdname checkdirs
#' @export

checkdir <-
  function(dir, subdir = TRUE, ...) {
    if (missing(dir))
      dir <- tcltk::tk_choose.dir()
    
    pdfs <-
      any(grepl("\\.pdf$", list.files(dir, recursive = subdir), ignore.case =
                  TRUE))
    htmls <-
      any(grepl(
        "\\.html?$",
        list.files(dir, recursive = subdir),
        ignore.case = TRUE
      ))
    
    if (pdfs)
      pdfres <- checkPDFdir(dir, subdir, ...)
    if (htmls)
      htmlres <- checkHTMLdir(dir, subdir, ...)
    
    if (pdfs & htmls) {
      if (!is.null(pdfres) & !is.null(htmlres))
        Res <- rbind(pdfres, htmlres)
      else
        stop("statcheck did not find any results")
      
    } else
      if (pdfs & !htmls) {
        if (!is.null(pdfres))
          Res <- pdfres
        else
          stop("statcheck did not find any results")
      }
    
    else
      if (!pdfs & htmls) {
        if (!is.null(htmlres))
          Res <- htmlres
        else
          stop("statcheck did not find any results")
      }
    
    else
      if (!pdfs & !htmls)
        stop("No PDF or HTML found")
    
    
    class(Res) <- c("statcheck", "data.frame")
    return(Res)
    
  }

process_stats <- function(test_type, test_stat, df1, df2, reported_p,
                          p_comparison, test_comparison, p_dec, test_dec,
                          OneTailedInTxt,
                          # options:
                          two_tailed, alpha, pZeroError, pEqualAlphaSig,
                          OneTailedTxt, OneTailedTests){
  
  # compute p-value ----------------------------------------------------------
  computed_p <- compute_p(test_type = test_type,
                          test_stat = test_stat,
                          df1 = df1,
                          df2 = df2,
                          two_tailed = two_tailed)
  
  # check if the result is an error ------------------------------------------
  error <- error_test(reported_p = reported_p, 
                      test_type = test_type, 
                      test_stat = test_stat,
                      df1 = df1,
                      df2 = df2,
                      p_comparison = p_comparison, 
                      test_comparison = test_comparison, 
                      p_dec = p_dec, 
                      test_dec = test_dec,
                      two_tailed = two_tailed,
                      alpha = alpha,
                      pZeroError = pZeroError)
  
  # check if the result is a decision error ----------------------------------
  
  if(!error){
    # if a result is not an error, it's automatically also  not a decision error
    decision_error <- FALSE
  } else {
    # only if a result is an error, it makes sense to check if it's also a 
    # decision error
    decision_error <- decision_error_test(reported_p = reported_p, 
                                          computed_p = computed_p,
                                          test_comparison = test_comparison,
                                          p_comparison = p_comparison,
                                          alpha = alpha, 
                                          pEqualAlphaSig = pEqualAlphaSig)
  }
  
  # correct for one-tailed tests in text ------------------------------------
  
  # OneTailedTxt is an option that determines whether statcheck should
  # try to correct for 1-tailed tests as follows:
  # if the phrase one-tailed/one-sided/directional is in the full text,
  # AND if the reported p-value would have been correct if it was a one-
  # tailed test, classify the result as consistent
  # don't apply this correction if OneTailedTests == FALSE, because this already
  # forces statcheck to treat all results as one-tailed tests
  if(OneTailedTxt == TRUE & OneTailedTests == FALSE){
    
    # select only results where the phrase "one-tailed", "one-sided" or
    # "directional" was mentioned in text, and that were an error when
    # we assumed two-tailed tests
    upForCorrection <- error & OneTailedInTxt
    
    # only start correction procedure if the result fits the criteria above
    if (upForCorrection){
      
      # for this case, recompute the p-value, but this time assuming a 
      # one-tailed test
      computed_p_1tail <- compute_p(test_type = test_type,
                                    test_stat = test_stat,
                                    df1 = df1,
                                    df2 = df2,
                                    two_tailed = FALSE)
      
      # check whether result would still be an error if 1-tailed
      error_1tail <- 
        error_test(reported_p = reported_p, 
                   test_type = test_type, 
                   test_stat = test_stat,
                   df1 = df1,
                   df2 = df2,
                   p_comparison = p_comparison, 
                   test_comparison = test_comparison, 
                   p_dec = p_dec, 
                   test_dec = test_dec,
                   two_tailed = FALSE,
                   alpha = alpha,
                   pZeroError = pZeroError)
      
      if(!error_1tail){
        # if a result is not an error, it's automatically also  not a decision error
        decision_error_1tail <- FALSE
      } else {
        # only if a result is an error, it makes sense to check if it's also a 
        # decision error
        decision_error_1tail <- 
          decision_error_test(reported_p = reported_p, 
                              computed_p = computed_p_1tail,
                              test_comparison = test_comparison,
                              p_comparison = p_comparison,
                              alpha = alpha, 
                              pEqualAlphaSig = pEqualAlphaSig)
      }
      
      # if the 1-tailed p-value is no longer an error, the original values of
      # error, decisionerror, and computed_p should be overwritten with the 1-
      # tailed versions
      if(error != error_1tail){
        computed_p <- computed_p_1tail
        error <- error_1tail
        decision_error <- decision_error_1tail
      }
    }    
  }
  
  result <- data.frame(computed_p = computed_p,
                       error = error,
                       decision_error = decision_error)
  
  return(result) 
}

error_test <- function(reported_p, test_type, test_stat,
                      df1, df2,
                      p_comparison, test_comparison, 
                      p_dec, test_dec, 
                      two_tailed,
                      alpha, pZeroError) {
  
  # replace 'ns' for > alpha -----------------------------------------------
  
  reported_p[p_comparison == "ns"] <- alpha
  p_comparison[p_comparison == "ns"] <- ">"
  
  # compute p-values -------------------------------------------------------
 
  # take into account that the reported test statistic may have been rounded
  # to that end, compute the upper and lower bound of the test statistic
  # based on the number of decimals that it was reported with. E.g., 
  # a t-value of 2.0 could have been rounded from anywhere between 1.95-2.05.
  
  if(test_stat >= 0){
    
    low_stat <- test_stat - (.5 / 10 ^ test_dec)
    up_stat <- test_stat + (.5 / 10 ^ test_dec)
  
    # switch around for negative test statistics
    } else if (test_stat < 0){
    low_stat <- test_stat + (.5 / 10 ^ test_dec)
    up_stat <- test_stat - (.5 / 10 ^ test_dec)
  }
  
  # Compute the p-values that belong to the upper and lower bound of the test
  # statistic. This is the range of p-values that would be correct.
  up_p <- compute_p(test_type = test_type,
                    test_stat = low_stat,
                    df1 = df1,
                    df2 = df2,
                    two_tailed = two_tailed)
  
  low_p <- compute_p(test_type = test_type,
                     test_stat = up_stat,
                     df1 = df1,
                     df2 = df2,
                     two_tailed = two_tailed)
  
  # p values smaller or equal to zero are errors ---------------------------
  
  if(pZeroError == TRUE & reported_p <= 0){
    error <- TRUE
    return(error)
  }
  
  # check errors for different combinations of <>= -------------------------
  
  if(test_comparison == "="){
    
    if(p_comparison == "="){
      
      error <- reported_p > round(up_p, p_dec) | reported_p < round(low_p, p_dec)
      return(error)
      
    } else if(p_comparison == "<"){
      
      error <- reported_p < low_p
      return(error)
      
    } else if(p_comparison == ">"){
      
      error <- reported_p > up_p
      return(error)
      
    }
    
  } else if(test_comparison == "<"){
    
    if(p_comparison == "="){
      error <- reported_p < round(up_p, p_dec)
      return(error)
      
    } else if(p_comparison == "<"){
    
      error <- reported_p < up_p
      return(error)
    } 
    else if(p_comparison == ">"){
      
      error <- FALSE
      return(error)
      
    }
    
  } else if(test_comparison == ">"){
    
    if(p_comparison == "="){
      
      error <- reported_p > round(low_p, p_dec)
      return(error)
      
    } else if(p_comparison == "<"){
      
      error <- FALSE
      return(error)
      
    } else if(p_comparison == ">"){
      
      error <- reported_p > low_p
      return(error)
      
    }
    
  }
  
  return(NA)
  
}

#' Extract statistics and recompute p-values
#' 
#' \code{statcheck} extracts Null Hypothesis Significance (NHST) results from 
#' strings and returns the extracted values, reported p-values and recomputed 
#' p-values. 
#' 
#' 
#' \code{statcheck} roughly works in three steps.
#' 
#' \strong{1. Scan text for statistical results}
#' 
#' \code{statcheck} uses regular expressions to recognizes statistical results 
#' from t-tests, F-tests, \eqn{\chi2}-tests, Z-tests, Q-tests, and correlations. 
#' statcheck can only recognize these results if the results are reported 
#' exactly according to the APA guidelines:
#' \itemize{
#'     \item \emph{t}(df) = value, \emph{p} = value
#'     \item \emph{F}(df1, df2) = value, \emph{p} = value
#'     \item \emph{r}(df) = value, p = value
#'     \item \emph{\eqn{\chi2}} (df, N = value) = value, \emph{p} = value 
#'     (N is optional)
#'     \item \emph{Z} = value, \emph{p} = value
#'     \item \emph{Q}(df) = value, \emph{p} = value (statcheck can distinguish
#'     between Q, Qw / Q-within, and Qb / Q-between)
#' }
#' \code{statcheck} takes into account that test statistics and p values may be 
#' exactly (=) or inexactly (< or >) reported. Different spacing has also been 
#' taken into account.
#' 
#' \strong{2. Recompute p-value}
#' 
#' \code{statcheck} uses the reported test statistic and degrees of freedom to
#' recompute the p-value. By default, the recomputed p-value is two-sided
#' 
#' \strong{3. Compare reported and recomputed p-value}
#' 
#' This comparison takes into account how the results were reported, e.g., 
#' p < .05 is treated differently than p = .05. Incongruent p values are marked 
#' as an \code{error}. If the reported result is significant and the recomputed 
#' result is not, or vice versa, the result is marked as a 
#' \code{decision_error}.
#' 
#' Correct rounding is taken into account. For instance, a reported t-value of 
#' 2.35 could correspond to an actual value of 2.345 to 2.354 with a range of 
#' p-values that can slightly deviate from the recomputed p-value. 
#' \code{statcheck} will not count cases like this as errors.
#' 
#' Note that when \code{statcheck} flags an \code{error} or 
#' \code{decision_error}, it implicitly assumes that the p-value is the 
#' inconsistent value, but it could just as well be the case that the test 
#' statistic or degrees of freedom contain a reporting error. \code{statcheck}
#' merely detects wether a set of numbers is consistent with each other.
#' 
#' @seealso
#' For more details, see the 
#' \href{https://rpubs.com/michelenuijten/statcheckmanual}{online manual}.
#' 
#' @param texts A vector of strings.
#' @param stat Specify which test types you want to extract. "t" to extract 
#' t-values, "F" to extract F-values, "cor" to extract correlations, "chisq"to 
#' extract \eqn{\chi2} values, "Z" to extract Z-values, and "Q" to extract 
#' Q-values. Using \code{c()} you can specify multiple tests. Defaults to all
#' tests.
#' @param OneTailedTests Logical. Do you want to assume that all reported tests 
#' are one-tailed (TRUE) or two-tailed (FALSE, default)?
#' @param alpha Assumed level of significance in the scanned texts. Defaults to 
#' .05.
#' @param pEqualAlphaSig Logical. If TRUE, statcheck counts p <= alpha as
#' significant (default), if FALSE, statcheck counts p < alpha as significant.
#' @param pZeroError Logical. If TRUE, statcheck counts p = .000 as an error 
#' (because a p-value is never exactly zero, and should be reported as < .001), 
#' if FALSE, statcheck does not count p = .000 automatically as an error.
#' @param OneTailedTxt Logical. If TRUE, statcheck searches the text for 
#' "one-sided", "one-tailed", and "directional" to identify the possible use of 
#' one-sided tests. If one or more of these strings is found in the text AND the 
#' result would have been correct if it was a one-sided test, the result is 
#' assumed to be indeed one-sided and is counted as correct.
#' @param AllPValues Logical. If TRUE, the output will consist of a dataframe 
#' with all detected p values, also the ones that were not part of the full 
#' results in APA format.
#' @param messages Logical. If TRUE, statcheck will print a progress bar while 
#' it's extracting statistics from text.
#' 
#' @return A data frame containing for each extracted statistic:
#' \describe{
#'     \item{source}{Name of the file of which the statistic is extracted}
#'     \item{test_type}{Character indicating the statistic that is extracted}
#'     \item{df1}{First degree of freedom (if applicable)}
#'     \item{df2}{Second degree of freedom}
#'     \item{test_comp}{Reported comparison of the test statistic, when 
#'     importing from pdf this will often not be converted properly}
#'     \item{test_value}{Reported value of the statistic}
#'     \item{p_comp}{Reported comparison, when importing from pdf this might not 
#'     be converted properly}
#'     \item{reported_p}{The reported p-value, or NA if the reported value was 
#'     n.s.}
#'     \item{computed_p}{The recomputed p-value}
#'     \item{raw}{Raw string of the statistical reference that is extracted}
#'     \item{error}{The computed p value is not congruent with the reported 
#'     p-value}
#'     \item{decision_error}{The reported result is significant whereas the 
#'     recomputed result is not, or vice versa.}
#'     \item{one_tailed_in_txt}{Logical. Does the text contain the string 
#'     "sided", "tailed", and/or "directional"?}
#'     \item{apa_factor}{What proportion of all detected p-values was part of a 
#'     fully APA reported result?}
#' }
#' 
#' @examples 
#' txt <- "blablabla the effect was very significant (t(100)=1, p < 0.001)"
#' statcheck(txt)
#' 
#' @export


statcheck <- function(texts,
                      stat = c("t", "F", "cor", "chisq", "Z", "Q"),
                      OneTailedTests = FALSE,
                      alpha = .05,
                      pEqualAlphaSig = TRUE,
                      pZeroError = TRUE,
                      OneTailedTxt = FALSE,
                      AllPValues = FALSE,
                      messages = TRUE){
  
  # We need empty data frames to store extracted statistics in
  # One for NHST results (Res) and one for p-values (pRes)
  Res <- data.frame(NULL)
  pRes <- data.frame(NULL)
  
  # to indicate where the statistics came from, we need a name for the input
  # texts. In some cases, this is the name of the file the text came from, but
  # if the text has no name, number them (with leading zeros to allow sorting)
  if (is.null(names(texts))){
    number_names <- seq_along(texts)
    max_length <- ceiling(log10(max(number_names)))
    names(texts) <- formatC(number_names, width = max_length, format = "d", flag = "0")
  }
  
  
  # start progress bar. If the argument messages == FALSE, don't print this 
  # progress bar. This is mainly useful for the unit tests; otherwise hundreds
  # of progress bars would be printed during testing and that makes the test 
  # results hard to read
  if(messages == TRUE){
    message("Extracting statistics...")
    pb <- utils::txtProgressBar(max = length(text), style = 3)
  }
  
  # for each text in the vector of input texts, extract all p-values and all
  # NHST results
  for (i in seq_along(texts)) {
    txt <- texts[i]
    
    # extract p-values ------------------------------------------
    
    # extract all p values. This is based on a pretty rough regular expression 
    # that will extract anything that resembles p =<> .... We need this info 
    # later on to calculate the APA factor: the ratio (statcheck results)/
    # (total # of p values). It is also possible to let statcheck return this
    # dataframe instead of the data frame with NHST results.
    pvalues <- extract_p_value(txt)
    
    # append and close:
    # in each repetition of the loop, the extracted p-values are appended 
    # to the existing pRes data frame, so it grows in each step
    if(nrow(pvalues) > 0){
      pvalues <- cbind(Source = names(txt), pvalues)
      
      pRes <- rbind(pRes, pvalues)
    }
    
    # after appending the pvalues dataframe to the main pRes dataframe,
    # the temporary dataframe pvalues can be removed. 
    rm(pvalues)
    
    # extract NHST results ------------------------------------------
    
    # extract all NHST results. This function scrapes the text for all APA 
    # reported NHST results and parses it so that the separate elements are
    # returned in one large dataframe
    nhst <- extract_stats(txt = txt,
                          stat = stat)
    
    # append and close: same logic as for the pvalues dataframe above
    if(nrow(nhst) > 0){
      
      nhst$Source <- names(txt)
      nhst$OneTailedInTxt <- extract_1tail(txt)
      
      Res <- rbind(Res, nhst)
      rownames(Res) <- NULL # to avoid having duplicated row names (1, 1, 2 etc.)
    }
    
    rm(nhst)
    
    # update the progress bar
    if(messages == TRUE){
      utils::setTxtProgressBar(pb, i)
    }
    
  }
  
  # close progress bar
  if(messages == TRUE){
    close(pb)
  }
  
  ###---------------------------------------------------------------------
  
  if (nrow(Res) > 0) {
    
    # If the argument OneTailedTests == TRUE, it forces statcheck to treat 
    # every encountered NHST result as a one-tailed test. Note: this is not the
    # same as the automated 1-tailed test detection (switched on with the 
    # argument: OneTailedTxt). The latter works more subtly (see comments in 
    # process_stats()). 
    if (OneTailedTests == TRUE) {
      two_tailed <- FALSE
    } else {
      two_tailed <- TRUE
    }
    
    # create empty variables to fill out during the loop
    Res$Computed <- rep(NA, nrow(Res))
    Res$Error <- rep(NA, nrow(Res))
    Res$DecisionError <- rep(NA, nrow(Res))
    
    # row by row, process the extracted statistics in Res. Specifically,
    # compute the p-value, check if the result is an error and a decision error,
    # and if indicated in the options, check & correct for 1-tailed tests
    for(i in seq_len(nrow(Res))){
      
      result <- process_stats(test_type = Res$Statistic[i],
                              test_stat = Res$Value[i],
                              df1 = Res$df1[i], 
                              df2 = Res$df2[i],
                              reported_p = Res$Reported.P.Value[i],
                              p_comparison = Res$Reported.Comparison[i],
                              test_comparison = Res$Test.Comparison[i],
                              p_dec = Res$dec[i],
                              test_dec = Res$testdec[i],
                              OneTailedInTxt = Res$OneTailedInTxt[i],
                              # options:
                              two_tailed = two_tailed,
                              alpha = alpha,
                              pZeroError = pZeroError,
                              pEqualAlphaSig = pEqualAlphaSig,
                              OneTailedTxt = OneTailedTxt,
                              OneTailedTests = OneTailedTests)
      
      Res$Computed[i] <- result$computed_p
      Res$Error[i] <- result$error
      Res$DecisionError[i] <- result$decision_error
    }
    
    ###---------------------------------------------------------------------
    
    # APAfactor: proportion of APA results (that statcheck reads) 
    # in total number of p values
    
    Res$APAfactor <- calc_APA_factor(pRes, Res)
    
    ###---------------------------------------------------------------------
    
    # select & reorder columns for final data frame
    Res <- Res[ , c("Source", "Statistic", "df1", "df2", "Test.Comparison",
                    "Value", "Reported.Comparison", "Reported.P.Value",
                    "Computed", "Raw", "Error", "DecisionError", 
                    "OneTailedInTxt", "APAfactor")]
    
    # rename columns based on the variable names in the script constants.R
    colnames(Res) <- c(VAR_SOURCE, VAR_TYPE, VAR_DF1, VAR_DF2, 
                       VAR_TEST_COMPARISON, VAR_TEST_VALUE, VAR_P_COMPARISON,
                       VAR_REPORTED_P, VAR_COMPUTED_P, VAR_RAW, VAR_ERROR, 
                       VAR_DEC_ERROR, VAR_1TAILTXT, VAR_APAFACTOR)
    
  }
  
  # Return ------------------------------------------------------------------
  
  if (AllPValues == FALSE) {
    
    # Return message when there are no results
    if (nrow(Res) > 0) {
      class(Res) <- c("statcheck", "data.frame")
      return(Res)
    } else {
      cat("statcheck did not find any results\n")
    }
    
  } else {
    
    if(nrow(pRes) > 0) {
      # rename columns based on the variable names in the script constants.R
      # first make sure that the columns are in the right order before renaming
      pRes <- pRes[, c("Source", "p_comp", "p_value", "p_dec")]
      colnames(pRes) <- c(VAR_SOURCE, VAR_P_COMPARISON, VAR_REPORTED_P, VAR_P_DEC)
      
      return(pRes)
    } else {
      cat("statcheck did not find any p-values\n")
    }
  }
}

context("Extract z-tests from string")

# test if the following z-tests are correctly retrieved ----------------------

# standard z-tests
test_that("z-tests are correctly parsed", {
  txt1 <- " z = 2.20, p = .03"
  
  result <- statcheck(txt1, messages = FALSE)
  
  expect_equal(nrow(result), 1)
  expect_equal(as.character(result[[VAR_TYPE]]), "Z")
  expect_true(is.na(result[[VAR_DF1]]))
  expect_true(is.na(result[[VAR_DF2]]))
  expect_equal(as.character(result[[VAR_TEST_COMPARISON]]), "=")
  expect_equal(result[[VAR_TEST_VALUE]], 2.2)
  expect_equal(as.character(result[[VAR_P_COMPARISON]]), "=")
  expect_equal(result[[VAR_REPORTED_P]], 0.03)
  expect_equal(as.character(result[[VAR_RAW]]), "z = 2.20, p = .03")
})

# standard z-tests in text
test_that("z-tests are retrieved from sentences", {
  txt1 <- "The effect was very significant, z = 2.20, p = .03."
  txt2 <- "Both effects were very significant, z = 2.20, p = .03, z = 1.23, p = .04."
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(nrow(result), 3)
  expect_equal(as.character(result[[VAR_SOURCE]]), c("1", "2", "2"))
})

# variation in spacing
test_that("z-tests with different spacing are retrieved from text", {
  txt1 <- " z = 2.20 , p = .03"
  txt2 <- " z=2.20,p=.03"
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(nrow(result), 2)
})

# variation in capitalization
test_that("upper case z-tests are retrieved from text", {
  txt <- " Z = 2.20 , p = .03"
  
  result <- statcheck(txt, messages = FALSE)
  
  expect_equal(nrow(result), 1)
})

# test if the following incorrect z-tests are not retrieved --------------------

# z test cannot have df
test_that("a z followed by degrees of freedom is not matched", {
  txt1 <- " z(28) = 2.20, p = .03"
  txt2 <- " Z(28) = 2.20, p = .03"
  
  expect_output(statcheck(c(txt1, txt2), messages = FALSE), "did not find any results")
  
})

context("check if inconsistencies/errors are correctly classified")

# test if the following cases are correctly identified as errors --------------

# check classification of regular errors in all types of tests
test_that("simple errors are classified as such", {
  txt1 <- "t(28) = 2.20, p = .03"
  txt2 <- "F(2, 28) = 2.20, p = .15"
  txt3 <- "r(28) = .22, p = .26"
  txt4 <- "chi2(28) = 22.20, p = .79"
  txt5 <- " z = 2.20, p = .04"
  txt6 <- "Q(28) = 22.20, p = .79"

  expect_true(statcheck(txt1, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt2, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt3, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt4, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt5, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt6, messages = FALSE)[[VAR_ERROR]])
})

# classify p-values of negative test statistics correctly
test_that("p-values of negative tests are correctly classified", {
  txt1 <- " Z = -2.42, p = 0.016" # no error
  txt2 <- "t(28) = -2.20, p = .03" # error
  
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt2, messages = FALSE)[[VAR_ERROR]])
  
})

# classify inexactly reported p-values correctly
test_that("inexactly reported p-values are correctly classified",{
  txt1 <- "t(28) = 2.20, ns"
  txt2 <- "t(28) = 2.20, p > .05"
  txt3 <- "t(28) = 2.0, p < .05"
  
  expect_true(statcheck(txt1, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt2, messages = FALSE)[[VAR_ERROR]])
  
  expect_false(statcheck(txt3, messages = FALSE)[[VAR_ERROR]])
})

# also classify decision errors as errors
test_that("decision errors are also classified as errors",{
  txt1 <- "t(28) = 1.20, p = .03"
  txt2 <- "t(28) = 2.20, p = .30"

  expect_true(statcheck(txt1, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt2, messages = FALSE)[[VAR_ERROR]])
})

# test if the following cases are correctly identified as correct -------------

# correct rounding
test_that("correctly rounded p-values are not considered errors", {
  txt1 <- "t(28) = 2, p = .02"
  txt2 <- "t(28) = 2, p = .14"
  txt3 <- "t(28) = 2.2, p = .03" # rounded lower bound p-value
  txt4 <- "t(28) = 2.2, p = .04"
  txt5 <- "t(28) = 2.20, p = .036"
  txt6 <- "t(28) = 2.20, p = .037"
  
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt2, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt4, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt5, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt6, messages = FALSE)[[VAR_ERROR]])
})

# test if different arguments concerning errors work --------------------------

# OneTailedTests: assume all tests are one-tailed
test_that("OneTailedTests considers everything as one-tailed", {
  txt1 <- "t(28) = 2.20, p = .02"
  txt2 <- "t(28) = 2.20, p = .04"
  txt3 <- "this test is one-tailed: t(28) = 2.20, p = .02, but this one is not: t(28) = 2.20, p = .04"

  expect_false(statcheck(txt1, messages = FALSE,  OneTailedTests = TRUE)[[VAR_ERROR]])
  expect_true(statcheck(txt2, messages = FALSE, OneTailedTests = TRUE)[[VAR_ERROR]])
  expect_equal(statcheck(txt3, messages = FALSE, OneTailedTests = TRUE)[[VAR_ERROR]], c(FALSE, TRUE))
})

# OneTailedTxt: automated detection of one-tailed test in text
test_that("automated one-tailed test detection works", {
  txt1 <- "t(28) = 2.20, p = .018"
  txt2 <- "t(28) = 2.20, p = .01, one-tailed"
  txt3 <- "t(28) = 2.20, p = .018, one-tailed"
  txt4 <- "t(28) = 2.20, p = .018, one-sided"
  txt5 <- "t(28) = 2.20, p = .018, directional"

  # don't correct for one-tailed testing here
  expect_true(statcheck(txt1, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt1, messages = FALSE, OneTailedTxt = TRUE)[[VAR_ERROR]])
  expect_true(statcheck(txt2, messages = FALSE, OneTailedTxt = TRUE)[[VAR_ERROR]])
  expect_true(statcheck(txt3, messages = FALSE)[[VAR_ERROR]])

  # correct for one-tailed testing here
  expect_false(statcheck(txt3, messages = FALSE, OneTailedTxt = TRUE)[[VAR_ERROR]])
  expect_false(statcheck(txt4, messages = FALSE, OneTailedTxt = TRUE)[[VAR_ERROR]])
  expect_false(statcheck(txt5, messages = FALSE, OneTailedTxt = TRUE)[[VAR_ERROR]])
  
  # check that p-values were corrected in these cases
  p_1tail <- pt(2.20, 28, lower.tail = FALSE)
  expect_equal(statcheck(c(txt3, txt4, txt5), messages = FALSE, 
                         OneTailedTxt = TRUE)[[VAR_COMPUTED_P]], rep(p_1tail, 3))
})

# pZeroError: check if p = .000 is counted as an inconsistency or not
test_that("you can adapt whether p = .000 is counted as inconsistent or not", {
  txt1 <- "t(28) = 22.20, p = .000"
  txt2 <- "t(28) = 22.20, p < .000" # this is always an Error

  expect_true(statcheck(txt1, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt1, messages = FALSE, pZeroError = FALSE)[[VAR_ERROR]])

  expect_true(statcheck(txt2, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt2, messages = FALSE, pZeroError = FALSE)[[VAR_ERROR]])
})

# test classifications of (in)exact test statistcs and (in)exact p-values ----

# test statistics exactly reported 
test_that("cases where t = ... are correctly classified", {
  
  # calculate range of correct p-values
  lowp <- pt(2.25, 28, lower.tail = FALSE)*2
  upp <- pt(2.15, 28, lower.tail = FALSE)*2
  
  # correct
  txt1 <- "t(28) = 2.2, p = .036" # correct
  txt2 <- "t(28) = 2.2, p < .08"  # correct
  txt3 <- "t(28) = 2.2, p > .02"  # correct
  
  # error
  txt4 <- paste("t(28) = 2.2, p >", upp)  # error
  txt5 <- paste("t(28) = 2.2, p <", lowp) # error
  
  txt6 <- "t(28) = 2.2, p = .08"  # error
  txt7 <- "t(28) = 2.2, p = .02"  # error
  txt8 <- "t(28) = 2.2, p > .08"  # error
  txt9 <- "t(28) = 2.2, p < .02"  # error
  
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt2, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE)[[VAR_ERROR]])
  
  expect_true(statcheck(txt4, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt5, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt6, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt7, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt8, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt9, messages = FALSE)[[VAR_ERROR]])
})


# test statistic reported as <
test_that("cases where t < ... are correctly classified", {
  
  # calculate range of correct p-values
  lowp <- pt(2.25, 28, lower.tail = FALSE)*2
  upp <- pt(2.15, 28, lower.tail = FALSE)*2
  
  # correct
  txt1 <- paste("t(28) < 2.20, p >", upp)
  txt2 <- "t(28) < 2.2, p = .08"
  txt3 <- "t(28) < 2.2, p > .08"
  txt4 <- "t(28) < 2.2, p < .08"
  txt5 <- "t(28) < 2.2, p > .02"
  
  # error
  txt6 <- paste("t(28) < 2.2, p =", lowp)
  txt7 <- paste("t(28) < 2.2, p <", lowp)
  txt8 <- "t(28) < 2.2, p < .02"
  txt9 <- "t(28) < 2.2, p = .02"
  
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt2, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt4, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt5, messages = FALSE)[[VAR_ERROR]])
  
  expect_true(statcheck(txt6, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt7, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt8, messages = FALSE)[[VAR_ERROR]])
  expect_true(statcheck(txt9, messages = FALSE)[[VAR_ERROR]])
  
})

# test statistic reported as >
test_that("cases where t > ... are correctly classified", {
  
  # calculate range of correct p-values
  lowp <- pt(2.25, 28, lower.tail = FALSE)*2
  upp <- pt(2.15, 28, lower.tail = FALSE)*2
  
  # correct
  txt1 <- paste("t(28) > 2.20, p <", upp)
  txt2 <- "t(28) > 2.2, p = .02"
  txt3 <- "t(28) > 2.2, p > .02"
  txt4 <- "t(28) > 2.2, p < .02"
  txt5 <- "t(28) > 2.2, p < .08"
  
  # error
  txt6 <- paste("t(28) > 2.2, p =", upp)
  txt7 <- paste("t(28) > 2.2, p >", upp)
  txt8 <- "t(28) > 2.2, p > .08"
  txt9 <- "t(28) > 2.2, p = .08"
  
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt2, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt4, messages = FALSE)[[VAR_ERROR]])
  expect_false(statcheck(txt5, messages = FALSE)[[VAR_ERROR]])
  
  expect_true(statcheck(txt6, messages = FALSE)[[VAR_ERROR]]) 
  expect_true(statcheck(txt7, messages = FALSE)[[VAR_ERROR]]) 
  expect_true(statcheck(txt8, messages = FALSE)[[VAR_ERROR]]) 
  expect_true(statcheck(txt9, messages = FALSE)[[VAR_ERROR]])
  
})

context("check if gross inconsistencies/decision errors are correctly classified")

# test if the following cases are correctly identified as decision errors -----

# check classification of regular decision errors in all types of tests
test_that("simple decision errors are classified as such", {
  txt1  <- "t(28) = 2.20, p = .06"
  txt2  <- "t(28) = 1.20, p = .03"
  
  txt3  <- "F(2, 28) = 22.20, p = .06"
  txt4  <- "F(2, 28) = 2.20, p = .03"
  
  txt5  <- "r(28) = .22, p = .03"
  txt6  <- "r(28) = .52, p = .06"
  
  txt7  <- "chi2(28) = 2.20, p = .03"
  txt8  <- "chi2(28) = 52.20, p = .06"
  
  txt9  <- " z = 1.20, p = .03"
  txt10 <- " z = 2.20, p = .06"
  
  txt11 <- "Q(28) = 2.20, p = .03"
  txt12 <- "Q(28) = 52.20, p = .06"
  
  expect_true(statcheck(txt1,  messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt2,  messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt3,  messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt4,  messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt5,  messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt6,  messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt7,  messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt8,  messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt9,  messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt10, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt11, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt12, messages = FALSE)[[VAR_DEC_ERROR]])
})

# test if different arguments concerning decision errors work -----------------

# alpha: detect decision errors for different alpha levels
test_that("decision errors are correctly classified when alpha levels are changed", {
  # all cases below are errors
  txt1 <- "t(28) = 2.20, p = .06"
  txt2 <- "t(28) = 1.20, p = .03"
  
  txt3 <- "t(28) = 2.20, p = .11"
  txt4 <- "t(28) = 1.20, p = .09"
  
  txt5 <- "t(28) = 5.20, p = .02"
  txt6 <- "t(28) = 1.20, p = .005"
  
  expect_true(statcheck(txt1, messages = FALSE, alpha = .05)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt2, messages = FALSE, alpha = .05)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt3, messages = FALSE, alpha = .10)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt4, messages = FALSE, alpha = .10)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt5, messages = FALSE, alpha = .01)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt6, messages = FALSE, alpha = .01)[[VAR_DEC_ERROR]])
})

# pEqualAlphaSig: should p = .05 be considered significant or not
test_that("decision errors are correctly classified when argument p = alpha is changed", {
  
  txt1 <- "t(28) = 2.20, p = .05"
  txt2 <- "t(28) = 2.20, p = .10"
  
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt1, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  # check pEqualAlphaSig for different alpha
  expect_false(statcheck(txt2, messages = FALSE, alpha = .10)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt2, messages = FALSE, pEqualAlphaSig = FALSE, 
                        alpha = .10)[[VAR_DEC_ERROR]])

})


# test classifications of (in)exact test statistcs and (in)exact p-values ----

# test statistics exactly reported & p-value exactly reported
test_that("cases where t = ..., p = ... are correctly classified", {
  
  # assume alpha = .05
  # calculate which t-values correspond to p-values of .04, .05, and .06
  t.04 <- qt(.04/2, 28, lower.tail = FALSE)
  t.05 <- qt(.05/2, 28, lower.tail = FALSE)
  t.06 <- qt(.06/2, 28, lower.tail = FALSE)
  
  txt1 <- paste0("t(28) = ", t.04, ", p = .04")
  txt2a <- paste0("t(28) = ", t.05 + .0001, ", p = .04") # approach computed p == .05
  txt2b <- paste0("t(28) = ", t.05 - .0001, ", p = .04") # approach computed p == .05
  txt3 <- paste0("t(28) = ", t.06, ", p = .04")
  
  txt4 <- paste0("t(28) = ", t.04, ", p = .05")
  txt5 <- paste0("t(28) = ", t.05, ", p = .05")
  txt6 <- paste0("t(28) = ", t.06, ", p = .05")
  
  txt7 <- paste0("t(28) = ", t.04, ", p = .06")
  txt8a <- paste0("t(28) = ", t.05 + .0001, ", p = .06") # approach computed p == .05
  txt8b <- paste0("t(28) = ", t.05 - .0001, ", p = .06") # approach computed p == .05
  txt9 <- paste0("t(28) = ", t.06, ", p = .06")
  
  # if pEqualSig == TRUE
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt2a, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt3, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt4, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt5, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt6, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt7, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt8a, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE)[[VAR_DEC_ERROR]])
  
  # if pEqualSig == FALSE
  expect_false(statcheck(txt1, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt2b, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt3, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt4, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt5, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt6, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt7, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt8b, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
})

# test statistics exactly reported & p-value < ...
test_that("cases where t = ..., p < ... are correctly classified", {
  
  # assume alpha = .05
  # calculate which t-values correspond to p-values of .04, .05, and .06
  t.04 <- qt(.04/2, 28, lower.tail = FALSE)
  t.05 <- qt(.05/2, 28, lower.tail = FALSE)
  t.06 <- qt(.06/2, 28, lower.tail = FALSE)
  
  txt1 <- paste0("t(28) = ", t.04 - .001, ", p < .04")   # approach computed p == .04
  txt2a <- paste0("t(28) = ", t.05 + .0001, ", p < .04") # approach computed p == .05
  txt2b <- paste0("t(28) = ", t.05 - .0001, ", p < .04") # approach computed p == .05
  txt3 <- paste0("t(28) = ", t.06, ", p < .04")
  
  txt4 <- paste0("t(28) = ", t.04, ", p < .05")
  txt5a <- paste0("t(28) = ", t.05 + .0001, ", p < .0500") # approach computed p == .05
  txt5b <- paste0("t(28) = ", t.05 - .0001, ", p < .0500") # approach computed p == .05
  txt6 <- paste0("t(28) = ", t.06, ", p < .05")
  
  txt7 <- paste0("t(28) = ", t.04, ", p < .06")
  txt8a <- paste0("t(28) = ", t.05 + .0001, ", p < .06") # approach computed p == .05
  txt8b <- paste0("t(28) = ", t.05 - .0001, ", p < .06") # approach computed p == .05
  txt9 <- paste0("t(28) = ", t.06, ", p < .06")
  
  # if pEqualSig == TRUE
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt2a, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt3, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt4, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt5a, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt6, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt7, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt8a, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE)[[VAR_DEC_ERROR]])
  
  # if pEqualSig == FALSE
  expect_false(statcheck(txt1, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt2b, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt3, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt4, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt5b, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt6, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt7, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt8b, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
})

# test statistics exactly reported & p-value > ...
test_that("cases where t = ..., p > ... are correctly classified", {
  
  # assume alpha = .05
  # calculate which t-values correspond to p-values of .04, .05, and .06
  t.04 <- qt(.04/2, 28, lower.tail = FALSE)
  t.05 <- qt(.05/2, 28, lower.tail = FALSE)
  t.06 <- qt(.06/2, 28, lower.tail = FALSE)
  
  txt1 <- paste0("t(28) = ", t.04 + .001, ", p > .04") # approach computed p == .04
  txt2a <- paste0("t(28) = ", t.05 + .0001, ", p > .04") # approach computed p == .05
  txt2b <- paste0("t(28) = ", t.05 - .0001, ", p > .04") # approach computed p == .05
  txt3 <- paste0("t(28) = ", t.06, ", p > .04")
  
  txt4 <- paste0("t(28) = ", t.04, ", p > .05")
  txt5a <- paste0("t(28) = ", t.05 + .0001, ", p > .05") # approach computed p == .05
  txt5b <- paste0("t(28) = ", t.05 - .0001, ", p > .05") # approach computed p == .05
  txt6 <- paste0("t(28) = ", t.06, ", p > .05")
  
  txt7 <- paste0("t(28) = ", t.04, ", p > .06")
  txt8a <- paste0("t(28) = ", t.05 + .0001, ", p > .06") # approach computed p == .05
  txt8b <- paste0("t(28) = ", t.05 - .0001, ", p > .06") # approach computed p == .05
  txt9 <- paste0("t(28) = ", t.06, ", p > .06")
  
  # if pEqualSig == TRUE
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt2a, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt4, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt5a, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt6, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt7, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt8a, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE)[[VAR_DEC_ERROR]])
  
  # if pEqualSig == FALSE
  expect_false(statcheck(txt1, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt2b, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt4, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt5b, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt6, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt7, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt8b, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
})

# test statistics < & p-value exactly reported
test_that("cases where t < ..., p = ... are correctly classified", {
  
  # assume alpha = .05
  # calculate which t-values correspond to p-values of .04, .05, and .06
  t.04 <- qt(.04/2, 28, lower.tail = FALSE)
  t.05 <- qt(.05/2, 28, lower.tail = FALSE)
  t.06 <- qt(.06/2, 28, lower.tail = FALSE)
  
  txt1 <- paste0("t(28) < ", t.04 - .001, ", p = .0400") # approach computed p == .04
  txt2 <- paste0("t(28) < ", t.05 - .0001, ", p = .04") # approach computed p == .05
  txt3 <- paste0("t(28) < ", t.06, ", p = .04")
  
  txt4 <- paste0("t(28) < ", t.04, ", p = .05")
  txt5 <- paste0("t(28) < ", t.05 - .001, ", p = .0500") # approach computed p == .05
  txt6 <- paste0("t(28) < ", t.06, ", p = .05")
  
  txt7 <- paste0("t(28) < ", t.04, ", p = .06")
  txt8 <- paste0("t(28) < ", t.05 - .0001, ", p = .0600") # approach computed p == .05
  txt9 <- paste0("t(28) < ", t.06 - .001, ", p = .0600") # approach computed p == .06
  
  # if pEqualSig == TRUE
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt2, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt3, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt4, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt5, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt6, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt7, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt8, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE)[[VAR_DEC_ERROR]])
  
  # if pEqualSig == FALSE
  expect_false(statcheck(txt1, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt2, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt3, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt4, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt5, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt6, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt7, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt8, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
})

# test statistics < & p-value <
test_that("cases where t < ..., p < ... are correctly classified", {
  
  # assume alpha = .05
  # calculate which t-values correspond to p-values of .04, .05, and .06
  t.04 <- qt(.04/2, 28, lower.tail = FALSE)
  t.05 <- qt(.05/2, 28, lower.tail = FALSE)
  t.06 <- qt(.06/2, 28, lower.tail = FALSE)
  
  txt1 <- paste0("t(28) < ", t.04, ", p < .04")
  txt2 <- paste0("t(28) < ", t.05 - .0001, ", p < .04") # approach computed p == .05
  txt3 <- paste0("t(28) < ", t.06, ", p < .04")
  
  txt4 <- paste0("t(28) < ", t.04, ", p < .05")
  txt5 <- paste0("t(28) < ", t.05 - .0001, ", p < .05") # approach computed p == .05
  txt6 <- paste0("t(28) < ", t.06, ", p < .05")
  
  txt7 <- paste0("t(28) < ", t.04, ", p < .06")
  txt8 <- paste0("t(28) < ", t.05 - .0001, ", p < .06") # approach computed p == .05
  txt9 <- paste0("t(28) < ", t.06, ", p < .06")
  
  # if pEqualSig == TRUE
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt2, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt3, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt4, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt5, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt6, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt7, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt8, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE)[[VAR_DEC_ERROR]])
  
  # if pEqualSig == FALSE
  expect_false(statcheck(txt1, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt2, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt3, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt4, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt5, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt6, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt7, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt8, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
})

# test statistics < & p-value >
test_that("cases where t < ..., p > ... are correctly classified", {
  
  # assume alpha = .05
  # calculate which t-values correspond to p-values of .04, .05, and .06
  t.04 <- qt(.04/2, 28, lower.tail = FALSE)
  t.05 <- qt(.05/2, 28, lower.tail = FALSE)
  t.06 <- qt(.06/2, 28, lower.tail = FALSE)
  
  txt1 <- paste0("t(28) < ", t.04, ", p > .04")
  txt2 <- paste0("t(28) < ", t.05 - .0001, ", p > .04") # approach computed p == .05
  txt3 <- paste0("t(28) < ", t.06, ", p > .04")
  
  txt4 <- paste0("t(28) < ", t.04, ", p > .05")
  txt5 <- paste0("t(28) < ", t.05 - .0001, ", p > .05") # approach computed p == .05
  txt6 <- paste0("t(28) < ", t.06, ", p > .05")
  
  txt7 <- paste0("t(28) < ", t.04, ", p > .06")
  txt8 <- paste0("t(28) < ", t.05 - .0001, ", p > .06") # approach computed p == .05
  txt9 <- paste0("t(28) < ", t.06, ", p > .06")
  
  # if pEqualSig == TRUE
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt2, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt4, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt5, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt6, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt7, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt8, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE)[[VAR_DEC_ERROR]])
  
  # if pEqualSig == FALSE
  expect_false(statcheck(txt1, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt2, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt4, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt5, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt6, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt7, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt8, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
})

# test statistics > & p-value exactly reported
test_that("cases where t > ..., p = ... are correctly classified", {
  
  # assume alpha = .05
  # calculate which t-values correspond to p-values of .04, .05, and .06
  t.04 <- qt(.04/2, 28, lower.tail = FALSE)
  t.05 <- qt(.05/2, 28, lower.tail = FALSE)
  t.06 <- qt(.06/2, 28, lower.tail = FALSE)
  
  txt1 <- paste0("t(28) > ", t.04 + .001, ", p = .0400") # approach computed p == .04
  txt2 <- paste0("t(28) > ", t.05 + .0001, ", p = .04") # approach computed p == .05
  txt3 <- paste0("t(28) > ", t.06, ", p = .04")
  
  txt4 <- paste0("t(28) > ", t.04, ", p = .05")
  txt5 <- paste0("t(28) > ", t.05 + .001, ", p = .0500") # approach computed p == .05
  txt6 <- paste0("t(28) > ", t.06, ", p = .05")
  
  txt7 <- paste0("t(28) > ", t.04, ", p = .06")
  txt8 <- paste0("t(28) > ", t.05 + .001, ", p = .0600") # approach computed p == .05
  txt9 <- paste0("t(28) > ", t.06 + .001, ", p = .0600") # approach computed p == .06
  
  # if pEqualSig == TRUE
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt2, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt4, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt5, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt6, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt7, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt8, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE)[[VAR_DEC_ERROR]])
  
  # if pEqualSig == FALSE
  expect_false(statcheck(txt1, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt2, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt4, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt5, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt6, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt7, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt8, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
})

# test statistics > & p-value <
test_that("cases where t > ..., p < ... are correctly classified", {
  
  # assume alpha = .05
  # calculate which t-values correspond to p-values of .04, .05, and .06
  t.04 <- qt(.04/2, 28, lower.tail = FALSE)
  t.05 <- qt(.05/2, 28, lower.tail = FALSE)
  t.06 <- qt(.06/2, 28, lower.tail = FALSE)
  
  txt1 <- paste0("t(28) > ", t.04 + .001, ", p < .0400") # approach computed p == .04
  txt2 <- paste0("t(28) > ", t.05 + .0001, ", p < .04") # approach computed p == .05
  txt3 <- paste0("t(28) > ", t.06, ", p < .04")
  
  txt4 <- paste0("t(28) > ", t.04, ", p < .05")
  txt5 <- paste0("t(28) > ", t.05 + .001, ", p < .0500") # approach computed p == .05
  txt6 <- paste0("t(28) > ", t.06, ", p < .05")
  
  txt7 <- paste0("t(28) > ", t.04, ", p < .06")
  txt8 <- paste0("t(28) > ", t.05 + .001, ", p < .0600") # approach computed p == .05
  txt9 <- paste0("t(28) > ", t.06 + .001, ", p < .0600") # approach computed p == .06
  
  # if pEqualSig == TRUE
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt2, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt4, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt5, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt6, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt7, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt8, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE)[[VAR_DEC_ERROR]])
  
  # if pEqualSig == FALSE
  expect_false(statcheck(txt1, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt2, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt4, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt5, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt6, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_false(statcheck(txt7, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt8, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
})

# test statistics > & p-value >
test_that("cases where t > ..., p > ... are correctly classified", {
  
  # assume alpha = .05
  # calculate which t-values correspond to p-values of .04, .05, and .06
  t.04 <- qt(.04/2, 28, lower.tail = FALSE)
  t.05 <- qt(.05/2, 28, lower.tail = FALSE)
  t.06 <- qt(.06/2, 28, lower.tail = FALSE)
  
  txt1 <- paste0("t(28) > ", t.04 + .001, ", p > .0400") # approach computed p == .04
  txt2 <- paste0("t(28) > ", t.05 + .0001, ", p > .04") # approach computed p == .05
  txt3 <- paste0("t(28) > ", t.06, ", p > .04")
  
  txt4 <- paste0("t(28) > ", t.04, ", p > .05")
  txt5 <- paste0("t(28) > ", t.05 + .001, ", p > .0500") # approach computed p == .05
  txt6 <- paste0("t(28) > ", t.06, ", p > .05")
  
  txt7 <- paste0("t(28) > ", t.04, ", p > .06")
  txt8 <- paste0("t(28) > ", t.05 + .001, ", p > .0600") # approach computed p == .05
  txt9 <- paste0("t(28) > ", t.06 + .001, ", p > .0600") # approach computed p == .06
  
  # if pEqualSig == TRUE
  expect_false(statcheck(txt1, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt2, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt4, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt5, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt6, messages = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt7, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt8, messages = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE)[[VAR_DEC_ERROR]])
  
  # if pEqualSig == FALSE
  expect_false(statcheck(txt1, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt2, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt3, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt4, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt5, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt6, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  
  expect_true(statcheck(txt7, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_true(statcheck(txt8, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
  expect_false(statcheck(txt9, messages = FALSE, pEqualAlphaSig = FALSE)[[VAR_DEC_ERROR]])
})

context("Extract t-tests from string")

# test if the following t-tests are correctly retrieved ----------------------

# standard t-test
test_that("t-tests are correctly parsed", {
  txt1 <- "t(28) = 2.20, p = .03"
  
  result <- statcheck(txt1, messages = FALSE)
  
  expect_equal(nrow(result), 1)
  expect_equal(as.character(result[[VAR_TYPE]]), "t")
  expect_true(is.na(result[[VAR_DF1]]))
  expect_equal(result[[VAR_DF2]], 28)
  expect_equal(as.character(result[[VAR_TEST_COMPARISON]]), "=")
  expect_equal(result[[VAR_TEST_VALUE]], 2.2)
  expect_equal(as.character(result[[VAR_P_COMPARISON]]), "=")
  expect_equal(result[[VAR_REPORTED_P]], 0.03)
  expect_equal(as.character(result[[VAR_RAW]]), "t(28) = 2.20, p = .03")
})

# standard t-tests in text
test_that("t-tests are retrieved from sentences", {
  txt1 <- "The effect was very significant, t(28) = 2.20, p = .03."
  txt2 <- "Both effects were very significant, t(28) = 2.20, p = .03, t(28) = 1.23, p = .04."
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(nrow(result), 3)
  expect_equal(as.character(result[[VAR_SOURCE]]), c("1", "2", "2"))
})

# variation in spacing
test_that("t-tests with different spacing are retrieved from text", {
  txt1 <- " t ( 28 ) = 2.20 , p = .03"
  txt2 <- "t(28)=2.20,p=.03"
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(nrow(result), 2)
})

# variations test statistic
test_that("variations in the t-statistic are retrieved from text", {
  txt1 <- "t(28) = -2.20, p = .03"
  txt2 <- "t(28) = 2,000.20, p = .03"
  txt3 <- "t(28) < 2.20, p = .03"
  txt4 <- "t(28) > 2.20, p = .03"
  txt5 <- "t(28) = %^&2.20, p = .03" # read as -2.20
  
  result <- statcheck(c(txt1, txt2, txt3, txt4, txt5), messages = FALSE)
  
  expect_equal(nrow(result), 5)
})

# variations p-value
test_that("variations in the p-value are retrieved from text", {
  txt1 <- "t(28) = 2.20, p = 0.03"
  txt2 <- "t(28) = 2.20, p < .03"
  txt3 <- "t(28) = 2.20, p > .03"
  txt4 <- "t(28) = 2.20, ns"
  txt5 <- "t(28) = 2.20, p = .5e-3"
  
  result <- statcheck(c(txt1, txt2, txt3, txt4, txt5), messages = FALSE)
  
  expect_equal(nrow(result), 5)
})

# corrected degrees of freedom
test_that("corrected degrees of freedom in t-tests are retrieved from text", {
  txt1 <- "t(28.1) = 2.20, p = .03"
  
  result <- statcheck(txt1, messages = FALSE)
  
  expect_equal(nrow(result), 1)
  expect_equal(result[[VAR_DF2]], 28.1)
})

# test if the following incorrect t-tests are not retrieved ------------------

# no test found
test_that("statcheck doesn't throw an error when input is missing", {
  expect_output(statcheck(NA, messages = FALSE), "did not find any results")
})

# punctuation
test_that("incorrect punctuation in t-tests are not retrieved from text", {
  txt1 <- "t(28) = 2.20; p = .03"
  txt2 <- "t[28] = 2.20, p = .03"
  txt3 <- "t(28) = .2.20, p = .03"
  
  expect_output(statcheck(c(txt1, txt2), messages = FALSE), "did not find any results")
})

test_that("capital t's are not retrieved from text", {
  txt1 <- "T(28) = 2.20, p = .03"
  
  expect_output(statcheck(txt1, messages = FALSE), "did not find any results")
})

# not a p-value
test_that("tests with 'p-values' larger than 1 are not retrieved from text", {
  txt1 <- "t(28) = 2.20, p = 1.03"
  
  expect_output(statcheck(txt1, messages = FALSE), "did not find any results")
})

# wrong df
test_that("t-tests with 2 dfs are not retrieved from text", {
  txt1 <- "t(2,28) = 2.20, p = .03"
  
  expect_output(statcheck(txt1, messages = FALSE), "did not find any results")
})

# weird encoding in minus sign followed by space
test_that("t-values with a weird minus sign and a space do not result in errors", {
    txt1 <- " t(553) = − 4.46, p < .0001" # this is an em dash or something
    
    expect_output(statcheck(txt1, messages = FALSE), "did not find any results")
})

# multiple comparison signs 
test_that("t-values with multiple comparison signs are not retrieved", {
  txt1 <- "t(38) >= 2.25, p = .03"
  txt2 <- "t(38) = 2.25, p >= .03"
  
  expect_output(statcheck(c(txt1, txt2), messages = FALSE), "did not find any results")
})

context("Extract correlations from string")

# test if the following correlations are correctly retrieved ----------------------

# standard correlation
test_that("correlations are correctly parsed", {
  txt1 <- "r(28) = .20, p = .03"
  
  result <- statcheck(txt1, messages = FALSE)
  
  expect_equal(nrow(result), 1)
  expect_equal(as.character(result[[VAR_TYPE]]), "r")
  expect_true(is.na(result[[VAR_DF1]]))
  expect_equal(result[[VAR_DF2]], 28)
  expect_equal(as.character(result[[VAR_TEST_COMPARISON]]), "=")
  expect_equal(result[[VAR_TEST_VALUE]], .2)
  expect_equal(as.character(result[[VAR_P_COMPARISON]]), "=")
  expect_equal(result[[VAR_REPORTED_P]], 0.03)
  expect_equal(as.character(result[[VAR_RAW]]), "r(28) = .20, p = .03")
})

# standard correlations in text
test_that("correlations are retrieved from sentences", {
  txt1 <- "The effect was very significant, r(28) = .20, p = .03."
  txt2 <- "Both effects were very significant, r(28) = .20, p = .03, r(28) = .23, p = .04."
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(nrow(result), 3)
  expect_equal(as.character(result[[VAR_SOURCE]]), c("1", "2", "2"))
})

# variation in spacing
test_that("correlations with different spacing are retrieved from text", {
  txt1 <- " r ( 28 ) = .20 , p = .03"
  txt2 <- "r(28)=.20,p=.03"
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(nrow(result), 2)
})

# test if the following incorrect correlations are not retrieved ------------------

# do not extract correlations larger than 1 or smaller than -1
test_that("correlations with impossible values are ignored", {
  txt1 <- "r(16) = 26.05, p = .10"
  txt2 <- "r(28) = −59, p = .0008"
  
  expect_output(statcheck(c(txt1, txt2), messages = FALSE), "did not find any results")
  
})

context("Extract p-values from string")

# test if the following p-values are correctly retrieved ----------------------

# standard p-values
test_that("p-values are correctly parsed", {
  txt1 <- "p = .05"
  txt2 <- "p < .05"
  txt3 <- "p > .05"
  
  result <- statcheck(c(txt1, txt2, txt3), messages = FALSE,
                        AllPValues = TRUE)
  
  expect_equal(nrow(result), 3)
  expect_equal(as.character(result[[VAR_P_COMPARISON]]), c("=", "<", ">"))
  expect_equal(result[[VAR_REPORTED_P]], rep(0.05, 3))
})

# non-significant results
test_that("results reported as ns are correctly parsed", {
  txt1 <- "the result was not significant, ns"
  
  result <- statcheck(txt1, messages = FALSE,
                      AllPValues = TRUE)
  
  expect_equal(nrow(result), 1)
  expect_equal(as.character(result[[VAR_P_COMPARISON]]), "ns")
  expect_true(is.na(result[[VAR_REPORTED_P]]))
})

# test if the following non p-values are not retrieved ------------------

# page number
test_that("page numbers are not extracted", {
  txt1 <- "see p. 01"
  
  expect_output(statcheck(txt1, messages = FALSE,
                          AllPValues = TRUE), "did not find any p-values")
})
  

context("Extract F-tests from string")

# test if the following F-tests are correctly retrieved ----------------------

# standard F-test
test_that("F-tests are correctly parsed", {
  txt1 <- "F(2, 28) = 2.20, p = .03"
  
  result <- statcheck(txt1, messages = FALSE)
  
  expect_equal(nrow(result), 1)
  expect_equal(as.character(result[[VAR_TYPE]]), "F")
  expect_equal(result[[VAR_DF1]], 2)
  expect_equal(result[[VAR_DF2]], 28)
  expect_equal(as.character(result[[VAR_TEST_COMPARISON]]), "=")
  expect_equal(result[[VAR_TEST_VALUE]], 2.2)
  expect_equal(as.character(result[[VAR_P_COMPARISON]]), "=")
  expect_equal(result[[VAR_REPORTED_P]], 0.03)
  expect_equal(as.character(result[[VAR_RAW]]), "F(2, 28) = 2.20, p = .03")
})

# standard F-tests in text
test_that("F-tests are retrieved from sentences", {
  txt1 <- "The effect was very significant, F(2, 28) = 2.20, p = .03."
  txt2 <- "Both effects were very significant, F(2, 28) = 2.20, p = .03, F(2, 28) = 1.23, p = .04."
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(nrow(result), 3)
  expect_equal(as.character(result[[VAR_SOURCE]]), c("1", "2", "2"))
})

# variation in spacing
test_that("F-tests with different spacing are retrieved from text", {
  txt1 <- " F ( 2 , 28 ) = 2.20 , p = .03"
  txt2 <- "F(2,28)=2.20,p=.03"
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(nrow(result), 2)
})

# variations in the degrees of freedom
test_that("corrected degrees of freedom in F-tests are retrieved from text", {
  txt1 <- "F(2.1, 28.1) = 2.20, p = .03"
  txt2 <- "F(l, 76) = 23.95, p <.001" # this wrong notation happens occasionally in articles
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(nrow(result), 2)
  expect_equal(result[[VAR_DF1]], c(2.1, 1))
  expect_equal(result[[VAR_DF2]], c(28.1, 76))
})

context("Extract Q-tests from string")

# test if the following Q-tests are correctly retrieved ----------------------

# standard Q-tests
test_that("Q-tests are correctly parsed", {
  txt1 <- "Q(2) = 2.20, p = .03"
  
  result <- statcheck(txt1, messages = FALSE)
  
  expect_equal(nrow(result), 1)
  expect_equal(as.character(result[[VAR_TYPE]]), "Q")
  expect_equal(result[[VAR_DF1]], 2)
  expect_true(is.na(result[[VAR_DF2]]))
  expect_equal(as.character(result[[VAR_TEST_COMPARISON]]), "=")
  expect_equal(result[[VAR_TEST_VALUE]], 2.2)
  expect_equal(as.character(result[[VAR_P_COMPARISON]]), "=")
  expect_equal(result[[VAR_REPORTED_P]], 0.03)
  expect_equal(as.character(result[[VAR_RAW]]), "Q(2) = 2.20, p = .03")
})

# standard Q-tests in text
test_that("Q-tests are retrieved from sentences", {
  txt1 <- "The effect was very significant, Q(2) = 2.20, p = .03."
  txt2 <- "Both effects were very significant, Q(2) = 2.20, p = .03, Q(2) = 1.23, p = .04."
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(nrow(result), 3)
  expect_equal(as.character(result[[VAR_SOURCE]]), c("1", "2", "2"))
})

# variation in spacing
test_that("Q-tests with different spacing are retrieved from text", {
  txt1 <- " Q ( 2 ) = 2.20 , p = .03"
  txt2 <- "Q(2)=2.20,p=.03"
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(nrow(result), 2)
})

# different types of Q-tests
test_that("different types of Q-tests are correctly parsed", {
  txt1 <- "Qw(2) = 2.20, p = .03"
  txt2 <- "Qwithin(2) = 2.20, p = .03"
  txt3 <- "Q-within(2) = 2.20, p = .03"
  
  txt4 <- "Qb(2) = 2.20, p = .03"
  txt5 <- "Qbetween(2) = 2.20, p = .03"
  txt6 <- "Q-between(2) = 2.20, p = .03"
  
  result <- statcheck(c(txt1, txt2, txt3, txt4, txt5, txt6), messages = FALSE)
  
  expect_equal(nrow(result), 6)
  expect_equal(as.vector(result[[VAR_TYPE]]), c(rep("Qw", 3), rep("Qb", 3)))
})

# test if the following 'incorrect' Q-tests are not retrieved ----------------
test_that("stats that only look like Q-tests are not retrieved", {
  # txt1 <- "q(2) = 2.2, p = .03" # lower case q is a different test
  txt2 <- "Qs(2) = 2.2, p = .03"
  txt3 <- "Qb(2, N = 187) = 2.20, p = .03"
  
  expect_output(statcheck(c(txt2, txt3), messages = FALSE), "did not find any results")
  
})

context("Calculate the APA factor")

test_that("correct APA factor is calculated", {
  
  txt1 <- "This text has 50% of its stats in APA style: t(28) = 2.20, p < .05, some other p = .035."
  txt2 <- "This text has 100% of its stats in APA style: t(28) = 2.20, p < .05."
  
  result1 <- statcheck(txt1, messages = FALSE)
  result2 <- statcheck(txt2, messages = FALSE)
  result12 <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(result1[[VAR_APAFACTOR]], .5)
  expect_equal(result2[[VAR_APAFACTOR]], 1)
  expect_equal(result12[[VAR_APAFACTOR]], c(.5, 1))
    
})

test_that("APA factor is calculated without problems if 1 source has no NHST", {
  
  txt1 <- "This text has 50% of its stats in APA style: t(28) = 2.20, p < .05, some other p = .035."
  txt2 <- "This text has 0% of its stats in APA style: p < .05."
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(result[[VAR_APAFACTOR]], .5)
  expect_equal(nrow(result), 1)
  
})

context("Extract chi2-tests from string")

# test if the following chi2-tests are correctly retrieved ----------------------

# standard chi2-test
test_that("chi2-tests are correctly parsed", {
  txt1 <- "chi2(28) = 2.20, p = .03"
  
  result <- statcheck(txt1, messages = FALSE)
  
  expect_equal(nrow(result), 1)
  expect_equal(as.character(result[[VAR_TYPE]]), "Chi2")
  expect_equal(result[[VAR_DF1]], 28)
  expect_true(is.na(result[[VAR_DF2]]))
  expect_equal(as.character(result[[VAR_TEST_COMPARISON]]), "=")
  expect_equal(result[[VAR_TEST_VALUE]], 2.2)
  expect_equal(as.character(result[[VAR_P_COMPARISON]]), "=")
  expect_equal(result[[VAR_REPORTED_P]], 0.03)
  expect_equal(as.character(result[[VAR_RAW]]), "i2(28) = 2.20, p = .03")
})

# standard chi2-tests in text
test_that("chi2-tests are retrieved from sentences", {
  txt1 <- "The effect was very significant, chi2(28) = 2.20, p = .03."
  txt2 <- "Both effects were very significant, chi2(28) = 2.20, p = .03, chi2(28) = 1.23, p = .04."
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(nrow(result), 3)
  expect_equal(as.character(result[[VAR_SOURCE]]), c("1", "2", "2"))
})

# variations in extracted "chi"
test_that("variations in spelling the Greek letter chi are picked up", {
  txt1 <- "X2(28) = 2.20, p = .03"
  txt2 <- "x2(28) = 2.20, p = .03"
  txt3 <- "chi_2(28) = 2.20, p = .03"
  
  result <- statcheck(c(txt1, txt2, txt3), messages = FALSE)
  
  expect_equal(nrow(result), 3)
})

# variation in degrees of freedom
test_that("different variations in df of chi2 are parsed correctly", {
  txt1 <- "chi2(28, N = 129) = 2.2, p = .03"
  txt2 <- "chi2(1, N = 11,455) = 16.78, p <.001"
  txt3 <- "chi2(1, n = 81) = 3.25, p = 0.07" # also extract lowercase n
  
  result <- statcheck(c(txt1, txt2, txt3), messages = FALSE)
  
  expect_equal(nrow(result), 3)
  
})

# variation in spacing
test_that("chi2-tests with different spacing are retrieved from text", {
  txt1 <- " chi2 ( 28 ) = 2.20 , p = .03"
  txt2 <- "chi2(28)=2.20,p=.03"
  
  result <- statcheck(c(txt1, txt2), messages = FALSE)
  
  expect_equal(nrow(result), 2)
})

#' @rdname checkdirs
#' @export

checkHTMLdir <- function(dir,
                         subdir = TRUE,
                         extension = TRUE,
                         ...) {
  if (missing(dir)) {
    dir <- tcltk::tk_choose.dir()
  }
  
  if (extension == TRUE) {
    pat = ".html|.htm"
  }
  
  if (extension == FALSE) {
    pat = ""
  }
  
  files <-
    list.files(dir,
               pattern = pat,
               full.names = TRUE,
               recursive = subdir)
  
  if (length(files) == 0) {
    stop("No HTML found")
  }
  
  txts <- character(length(files))
  message("Importing HTML files...")
  pb <- utils::txtProgressBar(max = length(files), style = 3)
  
  for (i in 1:length(files)) {
    txts[i] <-  getHTML(files[i])
    utils::setTxtProgressBar(pb, i)
  }
  
  close(pb)
  
  names(txts) <- basename(files)
  return(statcheck(txts, ...))
}

#' @rdname checkfiles
#' @export

checkPDF <-
  function(files, ...) {
    if (missing(files))
      files <- tcltk::tk_choose.files()
    
    txts <-  sapply(files, getPDF)
    names(txts) <-  basename(files)
    return(statcheck(txts, ...))
  }

#' Extract statistics from folders with PDF/HTML articles and recalculate 
#' p-values
#' 
#' These functions search for NHST results in all PDF and/or HTML articles in a 
#' certain folder and send the extracted statistics to \code{statcheck}.
#' 
#' @param dir String indicating the directory to be used. If this is left empty, 
#' a window will pop up from which you can choose a directory.
#' @param subdir Logical. Indicates whether you also want to check subfolders. 
#' Defaults to TRUE
#' @param extension Logical. Indicates whether the HTML extension should be 
#' checked. Defaults to TRUE
#' @param ... Arguments sent to \code{statcheck}.
#' 
#' @name checkdirs
#' 
#' @return A statcheck data frame with the extracted statistics. See 
#' \code{\link{statcheck}} for details.

NULL

#' Plot method for statcheck
#' 
#' Function for plotting of \code{statcheck} objects. Reported p values are 
#' plotted against recalculated p values, which allows the user to easily spot 
#' if articles contain miscalculations of statistical results.
#' 
#' If APAstyle = FALSE, inconsistencies between the reported and the recalculated p value are indicated with an orange dot. Recalculations of the p value that render a previously non significant result (p >= .5) as significant (p < .05), and vice versa, are considered decision errors, and are indicated with a red dot. Exactly reported p values (i.e. p = ..., as opposed to p < ... or p > ...) are indicated with a diamond.
#' 
#' @section Acknowledgements:
#' Many thanks to John Sakaluk who adapted the plot code to create graphs in 
#' APA style.
#' 
#' @seealso \code{\link{statcheck}}
#' 
#' @param x A statcheck object. See \code{\link{statcheck}}.
#' @param alpha assumed level of significance in the scanned texts. Defaults to 
#' .05.
#' @param APAstyle If TRUE, prints plot in APA style.
#' @param group Indicate grouping variable to facet plot. Only works when 
#' \code{APAstyle==TRUE}
#' @param ... arguments to be passed to methods, such as graphical parameters 
#' (see \code{\link{par}}).
#' 
#' @examples 
#' # First we need a statcheck object
#' # Here, we create one by running statcheck on some raw text
#' 
#' txt <- "This test is consistent t(28) = 0.2, p = .84, but this one is 
#' inconsistent: F(2, 28) = 4.2, p = .01. This final test is even a
#' gross/decision inconsistency: z = 1.23, p = .03"
#' 
#' result <- statcheck(txt)
#' 
#' # We can then plot the statcheck object 'result' by simply calling plot() on 
#' # "result". R will know what kind of plot to make, because "result" is of 
#' # class "statcheck"
#' plot(result)
#' 
#' @importFrom ggplot2 theme theme_bw element_blank element_line ggplot aes 
#' geom_point geom_vline geom_hline geom_abline annotate scale_x_continuous
#' scale_y_continuous scale_color_manual facet_grid
#' @importFrom rlang .data
#' @importFrom graphics plot.default points abline text par legend
#' 
#' @export

plot.statcheck <- function(
  x,
  alpha = .05,
  APAstyle = TRUE,
  group = NULL,
  ...
){
  
  # replace 'ns' for > alpha
  ns <- x[[VAR_P_COMPARISON]] == "ns"
  x[[VAR_P_COMPARISON]][ns] <- ">"
  x[[VAR_REPORTED_P]][ns] <- alpha
  
  if (APAstyle == TRUE) {
    
    # Add vector "Type" to statcheck object, specifying whether observations are
    # correctly reported, reporting inconsistencies, or decision errors.
    # First create an empty variable for Type to avoid a NOTE in the R CMD Check
    # that there is "no visible binding for global variable"
    Type <- rep(NA, nrow(x))
    x <- cbind(x, Type)
    
    x$Type[x[[VAR_ERROR]] == "FALSE" &
             x[[VAR_DEC_ERROR]] == "FALSE"] <- "Correctly Reported"
    x$Type[x[[VAR_ERROR]] == "TRUE" &
             x[[VAR_DEC_ERROR]] == "FALSE"] <- "Reporting Inconsistency"
    x$Type[x[[VAR_ERROR]] == "TRUE" &
             x[[VAR_DEC_ERROR]] == "TRUE"] <- "Decision Error"
    
    #Create ggplot "APA format" theme
    apatheme <- theme_bw() +
      theme(
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        axis.line = element_line()
      )
    
    #If no grouping variable is specified, don't use faceting
    if (is.null(group)) {
      #Create plot "p"; map computed p-values to x-axis, reported p-values to y-axis, and
      #color to the Type variable created earlier. Environment command allows apatheme to
      #be applied later because of bug when creating functions with ggplot2
      p <- ggplot(x,
                  aes(y = .data[[VAR_COMPUTED_P]],
                      x = .data[[VAR_REPORTED_P]],
                      col = Type),
                  environment = environment())
      
      #Add data points to plot
      p + geom_point(size = 2.5) +
        #Add vertical grey dashed line, located at specified alpha level
        geom_vline(xintercept = alpha,
                   color = "grey60",
                   linetype = "dashed") +
        #Add horizontal grey dashed line, located at specified alpha level
        geom_hline(yintercept = alpha,
                   color = "grey60",
                   linetype = "dashed") +
        #Add a line showing where accurately reported p-values should fall
        geom_abline(intercept = 0,
                    slope = 1,
                    color = "grey60") +
        #Add text annotations demarcating over-/under-estimated areas of the plot
        annotate("text",
                 x = 0.5,
                 y = .10,
                 label = "overestimated") +
        annotate("text",
                 x = 0.5,
                 y = .90,
                 label = "underestimated") +
        #Rename the x- and y-axis, and manually specify breaks
        scale_x_continuous(
          name = "Reported p-values",
          breaks = c(0.00, 0.05, 0.10, 0.25, 0.50, 0.75, 1.0),
          limits = c(0, 1)
        ) +
        scale_y_continuous(
          name = "Computed p-values",
          breaks = c(0.00, 0.05, 0.10, 0.25, 0.50, 0.75, 1.0),
          limits = c(0, 1)
        ) +
        #Manually specify greyscale colors for different levels of Type
        scale_color_manual(
          breaks = c(
            "Correctly Reported",
            "Reporting Inconsistency",
            "Decision Error"
          ),
          values = c("grey80", "black", "grey50")
        ) +
        apatheme
      
    } else {
      #If grouping variable is specified, use for faceting
      
      #Create plot "p"; map computed p-values to x-axis, reported p-values to y-axis, and
      #color to the Type variable created earlier. Environment command allows apatheme to
      #be applied later because of bug when creating functions with ggplot2
      p <- ggplot(x,
                  aes(y = rlang::.data[[VAR_COMPUTED_P]],
                      x = rlang::.data[[VAR_REPORTED_P]],
                      col = Type),
                  environment = environment())
      
      #Add data points to plot
      p + geom_point(size = 2.5) +
        #Add vertical grey dashed line, located at specified alpha level
        geom_vline(xintercept = alpha,
                   color = "grey60",
                   linetype = "dashed") +
        #Add horizontal grey dashed line, located at specified alpha level
        geom_hline(yintercept = alpha,
                   color = "grey60",
                   linetype = "dashed") +
        #Add a line showing where accurately reported p-values should fall
        geom_abline(intercept = 0,
                    slope = 1,
                    color = "grey60") +
        #Add text annotations demarcating over-/under-estimated areas of the plot
        annotate("text",
                 x = 0.5,
                 y = .10,
                 label = "overestimated") +
        annotate("text",
                 x = 0.5,
                 y = .90,
                 label = "underestimated") +
        #Rename the x- and y-axis, and manually specify breaks
        scale_x_continuous(name = "Reported p-values",
                           breaks = c(0.00, 0.05, 0.10, 0.25, 0.50, 0.75, 1.0)) +
        scale_y_continuous(name = "Computed p-values",
                           breaks = c(0.00, 0.05, 0.10, 0.25, 0.50, 0.75, 1.0)) +
        #Manually specify greyscale colors for different levels of Type
        scale_color_manual(
          breaks = c(
            "Correctly Reported",
            "Reporting Inconsistency",
            "Decision Error"
          ),
          values = c("grey80", "black", "grey50")
        ) +
        facet_grid(stats::as.formula(paste(group, "~ ."))) +
        apatheme
    }
    
  } else {
    # Extract limit args:
    args <- list(...)
    if (is.null(args$xlim))
      args$xlim <- c(0, 1)
    if (is.null(args$ylim))
      args$ylim <- c(0, 1)
    
    reported <- x[[VAR_REPORTED_P]]
    computed <- x[[VAR_COMPUTED_P]]
    
    # replace 'ns' for > alpha
    reported[x[[VAR_P_COMPARISON]] == "ns"] <- alpha
    
    # scatterplot of reported and recalculated p values
    do.call(plot.default, c(
      list(
        x = reported,
        y = computed,
        xlab = "reported p value",
        ylab = "recalculated p value",
        pch = 20
      ),
      args
    ))
    
    # orange dot for error
    points(reported[x[[VAR_ERROR]]],
           computed[x[[VAR_ERROR]]],
           pch = 20, col = "orange")
    
    # red dot for gross error (non-sig reported as sig and vice versa)
    points(reported[x[[VAR_DEC_ERROR]]],
           computed[x[[VAR_DEC_ERROR]]],
           pch = 20, col = "red")
    
    # indicate exact p values with diamond
    points(x[[VAR_REPORTED_P]][x[[VAR_P_COMPARISON]] == "="],
           computed[x[[VAR_P_COMPARISON]] == "="],
           pch = 5)
    
    # general layout of figure:
    # lines & text to indicate under- and overestimates
    abline(h = .05)
    abline(v = .05)
    abline(0, 1)
    
    text(.8, .4, "overestimated")
    text(.4, .8, "underestimated")
    
    text(0, .53, "non-sig", cex = .7)
    text(0, .50, "reported", cex = .7)
    text(0, .47, "as sig", cex = .7)
    
    text(.5, 0, "sig reported as non-sig", cex = .7)
    
    par(xpd = TRUE)
    legend(
      .88,
      -.15,
      pch = c(20, 20, 5),
      col = c("orange", "red", "black"),
      legend = c("p inconsistency", "decision error", "exact (p = ...)"),
      cex = .8
    )
    par(xpd = FALSE)
  }
}

calc_APA_factor <- function(pRes, Res){
  
  # select only the results of pRes that are from articles with at least 1 statcheck result
  pRes_selection <- pRes[pRes$Source %in% Res$Source, ]
  
  # Source should not be a factor. This would result in a bug down the road, if
  # one of the sources didn't have any APA results:
  # Error in by(Res_selection, Res_selection$Source, nrow)/by(pRes_selection,  : 
  # non-conformable arrays
  pRes_selection$Source <- as.vector(pRes_selection$Source)
  
  # select only the statcheck results that are from an article with at least one p value
  # this is relevant, because it sometimes happens that statcheck extracts less p values
  # p values than statcheck results. For instance in cases when a p value appears to be
  # greater than 1.
  
  Res_selection <-
    Res[Res$Source %in% pRes_selection$Source, ]
  APA <-
    by(Res_selection, Res_selection$Source, nrow) / by(pRes_selection, pRes_selection$Source, nrow)
  
  APAfactor <-
    round(as.numeric(apply(Res, 1, function(x)
      APA[which(names(APA) == x["Source"])])), 2)
  
  return(APAfactor)
  
}

# HTML TO TXT ------------------------------------------------------------------

getHTML <- function(x){
  
  strings <- lapply(x, function(fileName){
    con <- file(fileName)
    on.exit(close(con))
    raw_strings <- readChar(con, file.info(fileName)$size, useBytes = TRUE)
    return(raw_strings)
  })
  
  # Remove subscripts (except for p_rep)
  strings <- lapply(strings, gsub, pattern = "<sub>(?!rep).*?</sub>", replacement = "", perl = TRUE)
  
  # Remove HTML tags:
  strings <- lapply(strings, gsub, pattern = "<(.|\n)*?>", replacement = "")
  
  # encode everything in UTF-32 
  # this should ensure the same output accross multiple operating systems
  strings <- stringi::stri_enc_toutf32(strings) 
  
  # substitute a narrow no-break space (UTF-32 Decimal 8239) with a normal space 
  # (UTF-32 Decimal 61) [issue in JESP 2019 papers]
  strings <- lapply(strings, gsub, pattern = "8239",
                     replacement = "32", fixed = TRUE)
  
  # Revert back to UTF-8 encoding
  strings <- stringi::stri_enc_fromutf32(strings)
  
  # Replace html codes:
  # from: https://dev.w3.org/html5/html-author/charref 
  strings <- lapply(strings, gsub, pattern = "&#60;", replacement = "<", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&lt;", replacement = "<", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&LT;", replacement = "<", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#x0003C;", replacement = "<", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#x0003c;", replacement = "<", fixed = TRUE)
  
  strings <- lapply(strings, gsub, pattern = "&#61;", replacement = "=", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&equals;", replacement = "=", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#x0003D;", replacement = "=", fixed = TRUE)
  
  strings <- lapply(strings, gsub, pattern = "&#62;", replacement = ">", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&gt;", replacement = ">", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&GT;", replacement = ">", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#x0003E;", replacement = ">", fixed = TRUE)
  
  strings <- lapply(strings, gsub, pattern = "&#40;", replacement = "(", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#41;", replacement = ")", fixed = TRUE)
  
  strings <- lapply(strings, gsub, pattern = "&thinsp;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&nbsp;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&nnbsp;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#8239;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#x202F;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#32;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#160;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&ensp;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#8194;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&emsp;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#8195;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#8201;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&zwnj;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#8204;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&zwj;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#8205;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&lrm;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#8206;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&rlm;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#8207;", replacement = " ", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "\\s+", replacement = " ")
  
  strings <- lapply(strings, gsub, pattern = "\n", replacement = "")
  strings <- lapply(strings, gsub, pattern = "\r", replacement = "")
  
  
  strings <- lapply(strings, gsub, pattern = "&minus;", replacement = "-", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#x02212;", replacement = "-", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#8722;", replacement = "-", fixed = TRUE)
  
  strings <- lapply(strings, gsub, pattern = "&chi;", replacement = "X", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#x003C7;", replacement = "X", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#x003c7;", replacement = "X", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#967;", replacement = "X", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&Chi;", replacement = "X", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#x003A7;", replacement = "X", fixed = TRUE)
  strings <- lapply(strings, gsub, pattern = "&#935;", replacement = "X", fixed = TRUE)

  return(strings)
}

# PDF TO TXT -------------------------------------------------------------------
getPDF <- function(x){
  
  txtfiles <- character(length(x))
  for (i in 1:length(x)){
    
    system(paste('pdftotext -q -enc "ASCII7" "', x[i], '"', sep = ""))
    if (file.exists(gsub("\\.pdf$", "\\.txt", x[i]))) {
      fileName <- gsub("\\.pdf$", "\\.txt", x[i])
      strings <- readChar(fileName, file.info(fileName)$size)
      
      # remove carriage returns and new lines
      strings <- gsub(x = strings, pattern = "[\r\n]", replacement = "")
      
      # save result in vector
      txtfiles[i] <- strings
      
    } else{
      
      warning(paste("Failure in file", x[i]))
      txtfiles[i] <- ""
      
    }
  }
  
  return(txtfiles)
}

---
title: 'Report: statcheck results'
output: html_document
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, eval=TRUE, message=FALSE, warning=FALSE)
```

```{r}
library(statcheck)
pack <- sessionInfo()$otherPkgs 
info_statcheck <- pack[[which(names(pack)=="statcheck")]]
version <- info_statcheck$Version
date <- info_statcheck$Date
year <- strsplit(date,"-")[[1]][1]
```

*These results were automatically generated with the R package "statcheck" (Epskamp & Nuijten, `r year`), version `r version`.*

## Output statcheck

The table below reports the statcheck results of your manuscript. The table lists all Null Hypothesis Significance Tests that are reported according to the APA style, and indicates whether the reported p-value matches a recomputed p-value based on the reported test statistic and degrees of freedom. The first column provides green dots to indicate a consistent result, yellow dots for an inconsistent result not bearing on significance, and red dots to indicate an inconsistent result that bears on significance at the .05 level.

```{r}
setwd(system.file("rmd", package="statcheck"))
load("statcheckOutput.RData")
stat <- statcheckOutput

stat$Consistency[stat$Error==FALSE & stat$DecisionError==FALSE] <- "Consistent"
stat$Consistency[stat$Error==TRUE & stat$DecisionError==FALSE] <- "**Inconsistency**"
stat$Consistency[stat$Error==TRUE & stat$DecisionError==TRUE] <- "**Decision Inconsistency**"

stat_sparse <- stat[c("Source","Raw","Computed","Consistency")]

colnames(stat_sparse) <- c("Article","Result As Given In Text","Computed P-Value","Consistency")

##

# create color coding for the errors
red <- "<span style='color:red; font-size: 20pt'>$\\bullet$</span>"
yellow <- "<span style='color:orange; font-size: 20pt'>$\\bullet$</span>"
green <- "<span style='color:green; font-size: 20pt'>$\\bullet$</span>"

Code <- NA

Code[stat_sparse$Consistency == "**Decision Inconsistency**"] <- red
Code[stat_sparse$Consistency == "**Inconsistency**"] <- yellow
Code[stat_sparse$Consistency == "Consistent"] <- green

## 

stat_sparse <- cbind(Code,stat_sparse)

knitr::kable(stat_sparse)
```



***

## What is statcheck?

statcheck (Epskamp & Nuijten, `r year`) is an R package that automatically extracts statistical results from papers and checks the internal consistency of those results.

statcheck roughly works as follows:

1. Convert PDF or HTML to raw text
2. Use regular expressions to search for APA reported t-tests, F-tests, $\chi^2$-tests, Q-tests, Z-tests, and correlations.
3. Use reported test statistics and degrees of freedom to recalculate the p-value
4. Compare the reported p-value with the recalculated p-value
5. Flag inconsistent results as an Inconsistency
6. When the reported p-value is significant ($\alpha$ = .05) and the recalculated p-value is not, or vice versa, flag this result as a Decision Inconsistency (also sometimes identified as a "Gross Inconsistency")

statcheck takes into account one-sided testing as follows. If somewhere in the paper the words "one-sided", "one-tailed", or "directional" are mentioned, *and* the reported p-value would have been consistent if it was a one-sided test, statcheck counts it as a one-sided test and does not flag it as inconsistent.

## Interpretation

The variables in the table above can be interpreted as follows.

Variable | Interpretation
---------|-----------------------------------------------------------------------------
Code     | Color coding for inconsistencies. Green = Consistent, Yellow = Inconsistency, Red = Decision Inconsistency
Source   | The name of the file that was checked
Raw      | The full raw statistical result that was extracted
Computed | The recomputed p-value based on the reported test statistic and degrees of freedom
Consistency | Consistent = The reported p-value is consistent; Inconsistency = The reported p-value is not consistent; Decision Inconsistency = The reported p-value is not consistent and bears on significance ($\alpha$ = .05)

## Disclaimer

Please note that statcheck is an automated procedure and does not offer any explanations for detected inconsistencies (e.g., incorrect rounding, erroneous retrieval from computer output, a copy-paste error, or a a typo).

Also note that in the case of a flagged inconsistency, statcheck assumes that the p-value is the number that is misreported. However, it could well be the case that an inconsistent result is caused by a wrong test statistic or degrees of freedom.

For more details on what statcheck can and cannot do, and a list of common reasons why statcheck either does not find statistics or flags them as inconsistent, see the manual at <http://rpubs.com/michelenuijten/statcheckmanual>. 

***

## References
Nuijten, M.B., Hartgerink, C. H. J., van Assen, M. A. L. M., Epskamp, S., & Wicherts, J. M. (2016). The prevalence of statistical reporting errors in psychology (1985-2013). *Behavior Research Methods*, *48 (4)*, 1205-1226..  DOI: 10.3758/s13428-015-0664-2

Epskamp, S., & Nuijten, M. B. (`r year`). statcheck: Extract statistics from articles and recompute p-values. R package version `r version`.

\name{checkHTMLdir}

\alias{checkHTMLdir}

\title{Extract test statistics from all HTML files in a folder.}

\description{Extracts statistical references from a directory with HTML versions of articles. By default a GUI window is opened that allows you to choose the directory (using tcltk).}

\usage{checkHTMLdir(dir, subdir = TRUE, extension=TRUE, ...)}

\arguments{

  \item{dir}{String indicating the directory to be used.}

  \item{subdir}{Logical indicating whether you also want to check subfolders. Defaults to TRUE}
  
  \item{extension}{Logical, indicating whether the HTML extension should be checked. Defaults to TRUE}

  \item{\dots}{Arguments sent to  \code{\link{statcheck}}}

}

\details{See \code{\link{statcheck}} for more details. Use \code{\link{checkHTML}} to import individual HTML files. 

Note that the conversion to plain text and extraction of statistics can result in errors. Some statistical values can be missed, especially if the notation is unconventional. It is recommended to manually check some of the results.}

\value{A data frame containing for each extracted statistic:

\item{Source}{Name of the file of which the statistic is extracted}

\item{Statistic}{Character indicating the statistic that is extracted}

\item{df1}{First degree of freedom}

\item{df2}{Second degree of freedom (if applicable)}

\item{Test.Comparison}{Reported comparison of the test statistic, when importing from pdf this will often not be converted properly}

\item{Value}{Reported value of the statistic}

\item{Reported.Comparison}{Reported comparison, when importing from pdf this might not be converted properly}

\item{Reported.P.Value}{The reported p-value, or NA if the reported value was NS}

\item{Computed}{The recomputed p-value}

\item{Raw}{Raw string of the statistical reference that is extracted}

\item{Error}{The computed p value is not congruent with the reported p value}

\item{DecisionError}{The reported result is significant whereas the recomputed result is not, or vice versa.}

\item{OneTail}{Logical. Is it likely that the reported p value resulted from a correction for one-sided testing?}

\item{OneTailedInTxt}{Logical. Does the text contain the string "sided", "tailed", and/or "directional"?}

\item{CopyPaste}{Logical. Does the exact string of the extracted raw results occur anywhere else in the article?}}



\author{Sacha Epskamp <mail@sachaepskamp.com> & Michele B. Nuijten

<m.b.nuijten@uvt.nl>}







\seealso{\code{\link{statcheck}}, \code{\link{checkPDF}}, \code{\link{checkPDFdir}}, \code{\link{checkHTML}}, \code{\link{checkdir}}}

\examples{

  # with this command a menu will pop up from which you can select the directory with HTML articles

  # checkHTMLdir()



# you could also specify the directory beforehand

# for instance:

# DIR <- "C:/mydocuments/articles"

# checkHTMLdir(DIR)

}

\name{checkHTML}

\alias{checkHTML}

\title{Extract test statistics from HTML file.}

\description{Extracts statistical references from given HTML files.}

\usage{checkHTML(files, ...)}

\arguments{

  \item{files}{Vector of strings containing file paths to HTML files to check.}

  \item{\dots}{Arguments sent to  \code{\link{statcheck}}.}

}

\details{See \code{\link{statcheck}} for more details. Use \code{\link{checkHTMLdir}} to import al HTML files in a given directory at once. 

Note that the conversion to plain text and extraction of statistics can result in errors. Some statistical values can be missed, especially if the notation is unconvetional. It is recommended to manually check some of the results.}

\value{A data frame containing for each extracted statistic:

\item{Source}{Name of the file of which the statistic is extracted}

\item{Statistic}{Character indicating the statistic that is extracted}

\item{df1}{First degree of freedom}

\item{df2}{Second degree of freedom (if applicable)}

\item{Value}{Reported value of the statistic}

\item{Reported.Comparison}{Reported comparison, when importing from pdf this will often not be converted properly}

\item{Reported.P.Value}{The reported p-value, or NA if the reported value was NS}

\item{Computed}{The recomputed p-value}

\item{Raw}{Raw string of the statistical reference that is extracted}

\item{InExactError}{Error in inexactly reported p values as compared to the recalculated p values}

\item{ExactError}{Error in exactly reported p values as compared to the recalculated p values}

\item{DecisionError}{The reported result is significant whereas the recomputed result is not, or vice versa.}}



\author{Sacha Epskamp <mail@sachaepskamp.com> & Michele B. Nuijten

<m.b.nuijten@uvt.nl>}







\seealso{\code{\link{statcheck}}, \code{\link{checkPDF}}, \code{\link{checkPDFdir}}, \code{\link{checkHTMLdir}}, \code{\link{checkdir}}}

\examples{

  # given that my HTML file is called "article.html"

# and I saved it in "C:/mydocuments/articles"



    #checkHTML("C:/mydocuments/articles/article.html")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/identify.statcheck.R
\name{identify.statcheck}
\alias{identify.statcheck}
\title{Identify specific points in a statcheck plot.}
\usage{
\method{identify}{statcheck}(x, alpha = 0.05, ...)
}
\arguments{
\item{x}{A statcheck object. See \code{\link{statcheck}}.}

\item{alpha}{assumed level of significance in the scanned texts. Defaults to 
.05.}

\item{...}{arguments to be passed to methods, such as graphical parameters 
(see \code{\link{par}}).}
}
\description{
With this function you can simply point and click on the datapoints in the 
plot to see the corresponding statcheck details, such as the paper from which 
the data came and the exact statistical results.
}
\examples{
\dontrun{

# First we need a statcheck object
# Here, we create one by running statcheck on some raw text

txt <- "This test is consistent t(28) = 0.2, p = .84, but this one is 
inconsistent: F(2, 28) = 4.2, p = .01. This final test is even a
gross/decision inconsistency: z = 1.23, p = .03"

result <- statcheck(txt)

# Now, we can run identify.statcheck(), or shorter, simply identify():
identify(result)

# Further instructions:
# click on one or multiple points of interest
# press Esc
# a dataframe with information on the selected points will appear

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkHTML.R, R/checkPDF.R, R/doc-checkfiles.R
\name{checkHTML}
\alias{checkHTML}
\alias{checkPDF}
\alias{checkfiles}
\title{Extract statistics from PDF/HTML articles and recalculate p-values}
\usage{
checkHTML(files, ...)

checkPDF(files, ...)
}
\arguments{
\item{files}{Vector of strings containing file paths to HTML files to check.}

\item{...}{Arguments sent to \code{statcheck}.}
}
\value{
A statcheck data frame with the extracted statistics. See 
\code{\link{statcheck}} for details.
}
\description{
These functions search for NHST results in PDF and/or HTML articles and send 
the extracted statistics to \code{statcheck}.
}

\name{checkPDF}

\alias{checkPDF}

\title{Extract statistics and recompute p-values from pdf files.}

\description{Extracts statistical values (currently only t and F statistics) from PDF files. To this end the "pdftotext" program is used to convert PDF files to plain text files. This must be installed and PATH variables must be properly set so that this program can be used from command line.}

\usage{checkPDF(files, ...)}

\arguments{

  \item{files}{Vector with paths to the PDF files.}

  \item{\dots}{Arguments sent to  \code{\link{statcheck}}}

}

\details{See \code{\link{statcheck}} for more details. Use \code{\link{checkPDFdir}} to import every PDF file in a given directory. Currently only statistics in the form "(stat (df1, df2) = value, p = value)" are extracted.

Note that this function is still in devellopment. Some statistical values can be missed, especially if the notation is unconvetional. It is recommended to manually check some of the results.}

\value{A data frame containing for each extracted statistic:

\item{Source}{Name of the file of which the statistic is extracted}

\item{Statistic}{Character indicating the statistic that is extracted}

\item{df1}{First degree of freedom}

\item{df2}{Second degree of freedom (if applicable)}

\item{Value}{Reported value of the statistic}

\item{Reported.Comparison}{Reported comparison, when importing from pdf this will often not be converted properly}

\item{Reported.P.Value}{The reported p-value, or NA if the reported value was NS}

\item{Computed}{The recomputed p-value}

\item{Raw}{Raw string of the statistical reference that is extracted}

\item{InExactError}{Error in inexactly reported p values as compared to the recalculated p values}

\item{ExactError}{Error in exactly reported p values as compared to the recalculated p values}

\item{DecisionError}{The reported result is significant whereas the recomputed result is not, or vice versa.}}



\author{Sacha Epskamp <mail@sachaepskamp.com> & Michele B. Nuijten

<m.b.nuijten@uvt.nl>}







\seealso{\code{\link{statcheck}}, \code{\link{checkPDFdir}}}

\examples{

# given that my PDF file is called "article.pdf"

# and I saved it in "C:/mydocuments/articles"



 # checkPDF("C:/mydocuments/articles/article.pdf")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.statcheck.R
\name{plot.statcheck}
\alias{plot.statcheck}
\title{Plot method for statcheck}
\usage{
\method{plot}{statcheck}(x, alpha = 0.05, APAstyle = TRUE, group = NULL, ...)
}
\arguments{
\item{x}{A statcheck object. See \code{\link{statcheck}}.}

\item{alpha}{assumed level of significance in the scanned texts. Defaults to 
.05.}

\item{APAstyle}{If TRUE, prints plot in APA style.}

\item{group}{Indicate grouping variable to facet plot. Only works when 
\code{APAstyle==TRUE}}

\item{...}{arguments to be passed to methods, such as graphical parameters 
(see \code{\link{par}}).}
}
\description{
Function for plotting of \code{statcheck} objects. Reported p values are 
plotted against recalculated p values, which allows the user to easily spot 
if articles contain miscalculations of statistical results.
}
\details{
If APAstyle = FALSE, inconsistencies between the reported and the recalculated p value are indicated with an orange dot. Recalculations of the p value that render a previously non significant result (p >= .5) as significant (p < .05), and vice versa, are considered decision errors, and are indicated with a red dot. Exactly reported p values (i.e. p = ..., as opposed to p < ... or p > ...) are indicated with a diamond.
}
\section{Acknowledgements}{

Many thanks to John Sakaluk who adapted the plot code to create graphs in 
APA style.
}

\examples{
# First we need a statcheck object
# Here, we create one by running statcheck on some raw text

txt <- "This test is consistent t(28) = 0.2, p = .84, but this one is 
inconsistent: F(2, 28) = 4.2, p = .01. This final test is even a
gross/decision inconsistency: z = 1.23, p = .03"

result <- statcheck(txt)

# We can then plot the statcheck object 'result' by simply calling plot() on 
# "result". R will know what kind of plot to make, because "result" is of 
# class "statcheck"
plot(result)

}
\seealso{
\code{\link{statcheck}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statcheck-package.R
\docType{package}
\name{statcheck-package}
\alias{statcheck-package}
\title{statcheck: Extract statistics from articles and recompute p-values}
\description{
The package \code{statcheck} can extract Null Hypothesis Significance Test
(NHST) results from articles (or plain text) and recomputes p-values to check 
whether a reported NHST result is internally consistent or not.
}
\details{
\code{statcheck} can be used for multiple purposes, including:

\itemize{
    \item \strong{Self-checks}: you can use statcheck to make sure your 
    manuscript doesn't contain copy-paste errors or other inconsistencies 
    before you submit it to a journal.
    \item \strong{Peer review}: editors and reviewers can use statcheck to 
    check submitted manuscripts for statistical inconsistencies. They can ask 
    authors for a correction or clarification before publishing a manuscript.
    \item \strong{Research}: statcheck can be used to automatically extract 
    statistical test results from articles that can then be analyzed. You can 
    for instance investigate whether you can predict statistical 
    inconsistencies (see e.g., Nuijten et al., 2017 <doi:10.1525/collabra.102>), 
    or use it to analyze p-value distributions (see e.g., 
    Hartgerink et al., 2016 <doi:10.7717/peerj.1935>).
}
}
\section{Using statcheck on a string of text}{

The most basic usage of \code{statcheck} is to directly extract NHST results 
and check for inconsistencies in a string of text. See 
\code{\link{statcheck}} for details and an example of how to do this.
}

\section{Using statcheck on an article}{

Another option is to run \code{statcheck} on an article (PDF or HTML). This 
is a useful option if you want to check for inconsistencies in a single 
article (e.g., as a final check before you submit it). Depending on whether
you want to check an article in HTML or PDF, you can use 
\code{\link{checkHTML}} or \code{\link{checkPDF}}, respectively. Note: it is
recommended to check articles in HTML, as converting PDF files to plain text
sometimes results in some conversion errors.
}

\section{Using statcheck on a folder of articles}{

Finally, it is possible to run \code{statcheck} on an entire folder of 
articles. This is often useful for meta-research. To do so, you can use
\code{\link{checkPDFdir}} to check all PDF articles in a folder, 
\code{\link{checkHTMLdir}} to check all PDF articles in a folder, and 
\code{\link{checkdir}} to check both PDF and HTML articles in a folder.
}

\section{Accuracy of the algorithm in detecting inconsistencies}{

It is important to note that \code{statcheck} is not perfect. Its performance
in detecting NHST results depends on the type-setting and reporting style of 
an article and can vary widely. However, \code{statcheck} performs well in 
classifying the retrieved statistics in different consistency categories. We 
found that statcheck’s sensitivity (true positive rate) and specificity (true 
negative rate) were high: between 85.3% and 100%, and between 96.0% and 100%, 
respectively, depending on the assumptions and settings. The overall accuracy 
of statcheck ranged from 96.2% to 99.9%. More details on the validity study
can be found in \href{https://osf.io/preprints/psyarxiv/tcxaj/}{Nuijten et al., 2017}.
}

\section{Manual}{
 
Details on what statcheck can and cannot do, and how to install the package
and the necessary program Xpdf can be found in the 
\href{https://rpubs.com/michelenuijten/statcheckmanual}{online manual}.
}

\section{Web app}{

\code{statcheck} is also available as a free, online web app at 
\url{http://statcheck.io}.
}

\references{
Hartgerink, C. H. J., Van Aert, R. C. M., Nuijten, M. B., Wicherts, J. M., 
Van Assen, M. A. L. M. (2016). Distributions of p-values smaller than .05 in 
psychology: What is going on? \emph{PeerJ}, \emph{4}, e1935. 
doi: 10.7717/peerj.1935

Nuijten, M. B., Borghuis, J., Veldkamp, C. L. S., Dominguez-Alvarez, L., Van 
Assen, M. A. L. M., & Wicherts, J. M. (2017).  Journal data sharing policies 
and statistical reporting inconsistencies in psychology. 
\emph{Collabra: Psychology}, \emph{3}(1), 1-22. doi: 10.1525/collabra.102.

Nuijten, M. B., Van Assen, M. A. L. M., Hartgerink, C. H. J., Epskamp, S., & 
Wicherts, J. M. (2017). The validity of the tool "statcheck" in discovering 
statistical reporting inconsistencies. \emph{Preprint retrieved from 
https://osf.io/preprints/psyarxiv/tcxaj/.}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/MicheleNuijten/statcheck}
  \item Report bugs at \url{https://github.com/MicheleNuijten/statcheck/issues}
}

}
\author{
\strong{Maintainer}: Michele B. Nuijten \email{m.b.nuijten@uvt.nl} (\href{https://orcid.org/0000-0002-1468-8585}{ORCID})

Authors:
\itemize{
  \item Sacha Epskamp \email{mail@sachaepskamp.com} (\href{https://orcid.org/0000-0003-4884-8118}{ORCID})
}

Other contributors:
\itemize{
  \item Willem Sleegers (\href{https://orcid.org/0000-0001-9058-3817}{ORCID}) [contributor]
  \item Edoardo Costantini [contributor]
  \item Paul van der Laken (\href{https://orcid.org/0000-0002-0404-9114}{ORCID}) [contributor]
  \item Sean Rife (\href{https://orcid.org/0000-0002-6748-0841}{ORCID}) [contributor]
  \item John Sakaluk (\href{https://orcid.org/0000-0002-2515-9822}{ORCID}) [contributor]
  \item Chris Hartgerink (\href{https://orcid.org/0000-0003-1050-6809}{ORCID}) [contributor]
  \item Steve Haroz (\href{https://orcid.org/0000-0002-2725-9173}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statcheckReport.R
\name{statcheckReport}
\alias{statcheckReport}
\title{Generate HTML report for statcheck output}
\usage{
statcheckReport(statcheckOutput, outputFileName, outputDir)
}
\arguments{
\item{statcheckOutput}{statcheck output of one of the following functions: 
\code{\link{statcheck}}, \code{\link{checkPDFdir}}, \code{\link{checkPDF}}, 
\code{\link{checkHTMLdir}}, \code{\link{checkHTML}}, or
\code{\link{checkdir}}.}

\item{outputFileName}{String specifying the file name under which you want to 
save the generated HTML report. The extension ".html" is automatically added, 
so doesn't need to be specified in this argument.}

\item{outputDir}{String specifying the directory in which you want to save 
the generated HTML report.}
}
\value{
An HTML report, saved in the directory specified in the argument 
"outputDir".
}
\description{
This function uses R Markdown to generate a nicely formatted HTML report of 
\code{\link{statcheck}} output.
}
\details{
This function temporarily saves the inserted \code{statcheck} output as an 
.RData file in the "output" folder in the statcheck package directory. This 
file is then called by the .Rmd template that is saved in the folder "rmd", 
also in the statcheck package directory. After the HTML report is generated, 
the .RData file is removed again.
}
\examples{
\dontrun{

# first generate statcheck output, for instance by using the statcheck() 
function

txt <- "blablabla the effect was very significant (t(100)=1, p < 0.001)"
stat <- statcheck(txt)

# next, use this output to generate a nice HTML report of the results
statcheckReport(stat, outputFileName="statcheckHTMLReport", 
                outputDir="C:/mydocuments/results")

# you can now find your HTML report in the folder 
# "C:/mydocuments/results" under the name "statcheckHTMLReport.html".

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkHTMLdir.R, R/checkPDFdir.R, R/checkdir.R,
%   R/doc-checkdirs.R
\name{checkHTMLdir}
\alias{checkHTMLdir}
\alias{checkPDFdir}
\alias{checkdir}
\alias{checkdirs}
\title{Extract statistics from folders with PDF/HTML articles and recalculate 
p-values}
\usage{
checkHTMLdir(dir, subdir = TRUE, extension = TRUE, ...)

checkPDFdir(dir, subdir = TRUE, ...)

checkdir(dir, subdir = TRUE, ...)
}
\arguments{
\item{dir}{String indicating the directory to be used. If this is left empty, 
a window will pop up from which you can choose a directory.}

\item{subdir}{Logical. Indicates whether you also want to check subfolders. 
Defaults to TRUE}

\item{extension}{Logical. Indicates whether the HTML extension should be 
checked. Defaults to TRUE}

\item{...}{Arguments sent to \code{statcheck}.}
}
\value{
A statcheck data frame with the extracted statistics. See 
\code{\link{statcheck}} for details.
}
\description{
These functions search for NHST results in all PDF and/or HTML articles in a 
certain folder and send the extracted statistics to \code{statcheck}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statcheck-package.R, R/statcheck.R
\name{statcheck}
\alias{statcheck}
\title{Extract statistics and recompute p-values}
\usage{
statcheck(
  texts,
  stat = c("t", "F", "cor", "chisq", "Z", "Q"),
  OneTailedTests = FALSE,
  alpha = 0.05,
  pEqualAlphaSig = TRUE,
  pZeroError = TRUE,
  OneTailedTxt = FALSE,
  AllPValues = FALSE,
  messages = TRUE
)
}
\arguments{
\item{texts}{A vector of strings.}

\item{stat}{Specify which test types you want to extract. "t" to extract 
t-values, "F" to extract F-values, "cor" to extract correlations, "chisq"to 
extract \eqn{\chi2} values, "Z" to extract Z-values, and "Q" to extract 
Q-values. Using \code{c()} you can specify multiple tests. Defaults to all
tests.}

\item{OneTailedTests}{Logical. Do you want to assume that all reported tests 
are one-tailed (TRUE) or two-tailed (FALSE, default)?}

\item{alpha}{Assumed level of significance in the scanned texts. Defaults to 
.05.}

\item{pEqualAlphaSig}{Logical. If TRUE, statcheck counts p <= alpha as
significant (default), if FALSE, statcheck counts p < alpha as significant.}

\item{pZeroError}{Logical. If TRUE, statcheck counts p = .000 as an error 
(because a p-value is never exactly zero, and should be reported as < .001), 
if FALSE, statcheck does not count p = .000 automatically as an error.}

\item{OneTailedTxt}{Logical. If TRUE, statcheck searches the text for 
"one-sided", "one-tailed", and "directional" to identify the possible use of 
one-sided tests. If one or more of these strings is found in the text AND the 
result would have been correct if it was a one-sided test, the result is 
assumed to be indeed one-sided and is counted as correct.}

\item{AllPValues}{Logical. If TRUE, the output will consist of a dataframe 
with all detected p values, also the ones that were not part of the full 
results in APA format.}

\item{messages}{Logical. If TRUE, statcheck will print a progress bar while 
it's extracting statistics from text.}
}
\value{
A data frame containing for each extracted statistic:
\describe{
    \item{source}{Name of the file of which the statistic is extracted}
    \item{test_type}{Character indicating the statistic that is extracted}
    \item{df1}{First degree of freedom (if applicable)}
    \item{df2}{Second degree of freedom}
    \item{test_comp}{Reported comparison of the test statistic, when 
    importing from pdf this will often not be converted properly}
    \item{test_value}{Reported value of the statistic}
    \item{p_comp}{Reported comparison, when importing from pdf this might not 
    be converted properly}
    \item{reported_p}{The reported p-value, or NA if the reported value was 
    n.s.}
    \item{computed_p}{The recomputed p-value}
    \item{raw}{Raw string of the statistical reference that is extracted}
    \item{error}{The computed p value is not congruent with the reported 
    p-value}
    \item{decision_error}{The reported result is significant whereas the 
    recomputed result is not, or vice versa.}
    \item{one_tailed_in_txt}{Logical. Does the text contain the string 
    "sided", "tailed", and/or "directional"?}
    \item{apa_factor}{What proportion of all detected p-values was part of a 
    fully APA reported result?}
}
}
\description{
\code{statcheck} extracts Null Hypothesis Significance (NHST) results from 
strings and returns the extracted values, reported p-values and recomputed 
p-values.
}
\details{
\code{statcheck} roughly works in three steps.

\strong{1. Scan text for statistical results}

\code{statcheck} uses regular expressions to recognizes statistical results 
from t-tests, F-tests, \eqn{\chi2}-tests, Z-tests, Q-tests, and correlations. 
statcheck can only recognize these results if the results are reported 
exactly according to the APA guidelines:
\itemize{
    \item \emph{t}(df) = value, \emph{p} = value
    \item \emph{F}(df1, df2) = value, \emph{p} = value
    \item \emph{r}(df) = value, p = value
    \item \emph{\eqn{\chi2}} (df, N = value) = value, \emph{p} = value 
    (N is optional)
    \item \emph{Z} = value, \emph{p} = value
    \item \emph{Q}(df) = value, \emph{p} = value (statcheck can distinguish
    between Q, Qw / Q-within, and Qb / Q-between)
}
\code{statcheck} takes into account that test statistics and p values may be 
exactly (=) or inexactly (< or >) reported. Different spacing has also been 
taken into account.

\strong{2. Recompute p-value}

\code{statcheck} uses the reported test statistic and degrees of freedom to
recompute the p-value. By default, the recomputed p-value is two-sided

\strong{3. Compare reported and recomputed p-value}

This comparison takes into account how the results were reported, e.g., 
p < .05 is treated differently than p = .05. Incongruent p values are marked 
as an \code{error}. If the reported result is significant and the recomputed 
result is not, or vice versa, the result is marked as a 
\code{decision_error}.

Correct rounding is taken into account. For instance, a reported t-value of 
2.35 could correspond to an actual value of 2.345 to 2.354 with a range of 
p-values that can slightly deviate from the recomputed p-value. 
\code{statcheck} will not count cases like this as errors.

Note that when \code{statcheck} flags an \code{error} or 
\code{decision_error}, it implicitly assumes that the p-value is the 
inconsistent value, but it could just as well be the case that the test 
statistic or degrees of freedom contain a reporting error. \code{statcheck}
merely detects wether a set of numbers is consistent with each other.
}
\examples{
txt <- "blablabla the effect was very significant (t(100)=1, p < 0.001)"
statcheck(txt)

}
\seealso{
For more details, see the 
\href{https://rpubs.com/michelenuijten/statcheckmanual}{online manual}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trim.R
\name{trim}
\alias{trim}
\title{Trimming method for statcheck output}
\usage{
trim(object, ...)
}
\arguments{
\item{object}{a \code{statcheck} object.}

\item{...}{additional arguments affecting the trimmed output.}
}
\value{
A data frame containing for each source of statistics:
\describe{
    \item{source}{Name of the file/origin of which the statistics are 
    extracted}
    \item{raw}{Raw string of the statistical reference that is extracted}
    \item{computed_p}{The recomputed p-value}
    \item{error}{The computed p value is not congruent with the reported 
    p-value}
    \item{decision_error}{The reported result is significant whereas the 
    recomputed result is not, or vice versa.}
}
}
\description{
Returns a subset of columns of a \code{statcheck} object.
}
\examples{
txt <- "blablabla the effect was very significant (t(100)=1, p < 0.001)"
stat <- statcheck(txt)
trim(stat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.statcheck.R
\name{summary.statcheck}
\alias{summary.statcheck}
\title{Summary method for statcheck}
\usage{
\method{summary}{statcheck}(object, ...)
}
\arguments{
\item{object}{a \code{statcheck} object.}

\item{...}{additional arguments affecting the summary produced.}
}
\value{
A data frame containing for each source of statistics:
\describe{
    \item{source}{Name of the file/origin of which the statistics are 
    extracted}
    \item{nr_p_values}{The number of extracted reported p values per article}
    \item{nr_errors}{The number of errors per article}
    \item{nr_decision_errors}{The number of decision errors per article}
}
}
\description{
Gives the summaries for a \code{statcheck} object.
}
\examples{
txt <- "blablabla the effect was very significant (t(100)=1, p < 0.001)"
stat <- statcheck(txt)
summary(stat)

}
